import unittest


class ModulesImportTestCase(unittest.TestCase):
    """Simple test cases, verifying core modules were installed properly."""

    def test_hashlib_sha3(self):
        import hashlib
        self.assertIsNotNone(hashlib.sha3_512())

    def test_pyetheroll(self):
        from pyetheroll.etheroll import Etheroll
        self.assertTrue(hasattr(Etheroll, 'player_roll_dice'))


if __name__ == '__main__':
    unittest.main()



from kivy.clock import Clock
from kivy.properties import StringProperty
from kivy.uix.boxlayout import BoxLayout

from etherollapp.etheroll.settings import Settings
from etherollapp.etheroll.ui_utils import load_kv_from_py

load_kv_from_py(__file__)


class ImportKeystore(BoxLayout):
    keystore_path = StringProperty()

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        Clock.schedule_once(self.load_keystore_path)

    def load_keystore_path(self, dt=None):
        """Updates keystore path displayed in the UI."""
        self.keystore_path = Settings.get_keystore_path()



import unittest
from unittest import mock

from etherollapp.service import utils


class TestUtils(unittest.TestCase):

    @staticmethod
    def patch_platform():
        return mock.patch('etherollapp.service.utils.platform', 'android')

    @staticmethod
    def patch_jnius(m_jnius=mock.MagicMock()):
        return mock.patch.dict('sys.modules', jnius=m_jnius)

    def test_start_roll_polling_service(self):
        """
        Call `start_roll_polling_service()` with and without args.
        Verifies the PythonActivity is being created and started.
        """
        # test without args
        m_jnius = mock.MagicMock()
        with self.patch_platform(), self.patch_jnius(m_jnius):
            utils.start_roll_polling_service()
        assert m_jnius.autoclass().start.call_args_list == [
            mock.call(m_jnius.autoclass().mActivity, 'null')
        ]
        # test with args
        m_jnius = mock.MagicMock()
        arguments = {'foo': 'bar', 'foobar': True}
        with self.patch_platform(), self.patch_jnius(m_jnius):
            utils.start_roll_polling_service(arguments)
        assert m_jnius.autoclass.call_args_list == [
            mock.call('com.github.andremiras.etheroll.ServiceService'),
            mock.call('org.kivy.android.PythonActivity'),
        ]
        assert m_jnius.autoclass().start.call_args_list == [
            mock.call(
                m_jnius.autoclass().mActivity,
                '{"foo": "bar", "foobar": true}')
        ]


if __name__ == '__main__':
    unittest.main()



import os

from kivy.clock import Clock, mainthread
from kivy.properties import StringProperty
from kivy.uix.boxlayout import BoxLayout

from etherollapp.etheroll.ui_utils import SubScreen, load_kv_from_py
from etherollapp.etheroll.utils import StringIOCBWrite, run_in_thread
from etherollapp.version import __version__

load_kv_from_py(__file__)


class AboutOverview(BoxLayout):
    project_page_property = StringProperty(
        "https://github.com/AndreMiras/EtherollApp")
    about_text_property = StringProperty()

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        Clock.schedule_once(lambda dt: self.load_about())

    def load_about(self):
        self.about_text_property = "" + \
            "EtherollApp version: %s\n" % (__version__) + \
            "Project source code and info available on GitHub at:\n" + \
            "[color=00BFFF][ref=github]" + \
            self.project_page_property + \
            "[/ref][/color]"


class AboutChangelog(BoxLayout):
    changelog_text_property = StringProperty()

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        Clock.schedule_once(lambda dt: self.load_changelog())

    @staticmethod
    def src_dir():
        return os.path.join(os.path.dirname(os.path.abspath(__file__)), '..')

    def load_changelog(self):
        changelog_path = os.path.join(self.src_dir(), 'CHANGELOG.md')
        with open(changelog_path, 'r') as f:
            self.changelog_text_property = f.read()
        f.close()


class AboutDiagnostic(BoxLayout):
    stream_property = StringProperty()

    @mainthread
    def callback_write(self, s):
        """Updates the UI with test progress."""
        self.stream_property += s

    @run_in_thread
    def run_tests(self):
        """Loads the test suite and hook the callback for progress report."""
        # lazy loading
        import unittest
        from etherollapp.testsuite import suite
        test_suite = suite()
        self.stream_property = ""
        stream = StringIOCBWrite(callback_write=self.callback_write)
        verbosity = 2
        unittest.TextTestRunner(
                stream=stream, verbosity=verbosity).run(test_suite)


class AboutScreen(SubScreen):
    pass



from kivy.app import App
from kivy.clock import Clock, mainthread
from kivy.properties import ListProperty
from kivymd.label import MDLabel
from kivymd.list import ILeftBody, ThreeLineAvatarListItem
from pyetheroll.constants import ROUND_DIGITS

from etherollapp.etheroll.ui_utils import Dialog, SubScreen, load_kv_from_py
from etherollapp.etheroll.utils import run_in_thread

load_kv_from_py(__file__)


class DiceResultWidget(ILeftBody, MDLabel):
    pass


class RollResultsScreen(SubScreen):

    roll_logs = ListProperty()

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        # TODO: make it a property that starts/stops spinner on set
        self._fetching_results = False
        Clock.schedule_once(self._after_init)

    def _after_init(self, dt):
        """Binds events."""
        self.update_roll_list()

    # TODO: create a dedicated PullRefreshScrollView that handle all this
    def on_scroll_stop(self, scroll_y):
        """Refreshs roll results on overscroll."""
        # refresh if we hit the bottom or if we over pull
        # currently hitting the bottom will refresh last transactions only
        # rather than loading more ancient transactions
        if scroll_y == 0 or scroll_y >= 1.03:
            if not self._fetching_results:
                self.get_last_results()

    def toggle_spinner(self, show):
        spinner = self.ids.spinner_id
        spinner.toggle(show)

    @property
    def pyetheroll(self):
        """
        We want to make sure we go through the `Controller.pyetheroll` property
        each time, because it recreates the Etheroll object on chain_id
        changes.
        """
        controller = App.get_running_app().root
        return controller.pyetheroll

    @mainthread
    def on_roll_logs(self, instance, value):
        """Updates UI using roll results list."""
        self.update_roll_list()

    @staticmethod
    @mainthread
    def on_connection_refused():
        title = 'No network'
        body = 'No network, could not retrieve roll history.'
        dialog = Dialog.create_dialog(title, body)
        dialog.open()

    @run_in_thread
    def get_last_results(self):
        """
        Gets last rolls & results using pyetheroll lib and updates `roll_logs`
        list property.
        """
        # lazy loading
        from etherscan.client import ConnectionRefused
        controller = App.get_running_app().root
        account = controller.current_account
        if not account:
            controller.on_account_none()
            self.on_back()
            return
        self._fetching_results = True
        self.toggle_spinner(show=True)
        address = "0x" + account.address.hex()
        try:
            self.roll_logs = self.pyetheroll.get_merged_logs(
                address=address)
        except ConnectionRefused:
            self.on_connection_refused()
        self.toggle_spinner(show=False)
        self._fetching_results = False

    @staticmethod
    def create_item_from_dict(roll_log):
        """Creates a roll list item from a roll log dictionary."""
        # lazy loading
        from pyetheroll.utils import EtherollUtils
        bet_log = roll_log['bet_log']
        bet_result = roll_log['bet_result']
        bet_value_ether = bet_log['bet_value_ether']
        roll_under = bet_log['roll_under']
        date_time = bet_log['datetime']
        # will keep default value on unresolved bets
        dice_result = '?'
        player_won = None
        sign = '?'
        win_color = (0, 1, 0, 1)
        loss_color = (1, 0, 0, 1)
        unresolved_color = (0.5, 0.5, 0.5, 1)
        text_color = unresolved_color
        profit_loss_str = '?'
        # resolved bets case
        if bet_result is not None:
            dice_result = bet_result['dice_result']
            player_won = dice_result < roll_under
            sign = '<' if player_won else '>'
            text_color = win_color if player_won else loss_color
            chances_win = roll_under - 1
            profit = EtherollUtils.compute_profit(bet_value_ether, chances_win)
            profit_loss = profit if player_won else -bet_value_ether
            profit_loss_str = (
                '{profit_loss:+.{round_digits}f}'
            ).format(**{
                'profit_loss': profit_loss,
                'round_digits': ROUND_DIGITS})
        text = ('{0} ETH').format(profit_loss_str)
        secondary_text = '{0} {1} {2}'.format(
            dice_result, sign, roll_under)
        tertiary_text = date_time.strftime("%Y-%m-%d %H:%M:%S")
        # tertiary_text is in fact embedded in secondary_text with new line
        secondary_text += '\n' + tertiary_text
        avatar_sample_widget = DiceResultWidget(
            text=str(dice_result), font_style='Title',
            theme_text_color='Custom', text_color=text_color)
        list_item = ThreeLineAvatarListItem(
            text=text, secondary_text=secondary_text)
        list_item.add_widget(avatar_sample_widget)
        return list_item

    def update_roll_list(self):
        """Updates the roll results list widget."""
        roll_list = self.ids.roll_list_id
        roll_list.clear_widgets()
        roll_logs = reversed(self.roll_logs)
        for roll_log in roll_logs:
            list_item = self.create_item_from_dict(roll_log)
            roll_list.add_widget(list_item)



from kivy.properties import StringProperty
from kivy.uix.scrollview import ScrollView

from etherollapp.etheroll.ui_utils import load_kv_from_py

load_kv_from_py(__file__)


class ScrollableLabel(ScrollView):
    """https://github.com/kivy/kivy/wiki/Scrollable-Label"""
    text = StringProperty('')



#!/usr/bin/env python
from eth_utils import to_checksum_address
from kivy.app import App
from kivy.clock import Clock, mainthread
from kivy.logger import Logger
from kivy.properties import ObjectProperty, StringProperty
from kivy.uix.floatlayout import FloatLayout
from kivy.utils import platform
from kivymd.bottomsheet import MDListBottomSheet
from kivymd.theming import ThemeManager
from raven import Client
from requests.exceptions import ConnectionError

from etherollapp.etheroll.constants import API_KEY_PATH
from etherollapp.etheroll.flashqrcode import FlashQrCodeScreen
from etherollapp.etheroll.settings import Settings
from etherollapp.etheroll.settings_screen import SettingsScreen
from etherollapp.etheroll.switchaccount import SwitchAccountScreen
from etherollapp.etheroll.ui_utils import Dialog, load_kv_from_py
from etherollapp.etheroll.utils import run_in_thread
from etherollapp.osc.osc_app_server import OscAppServer
from etherollapp.sentry_utils import configure_sentry
from etherollapp.service.utils import start_roll_polling_service

load_kv_from_py(__file__)


class Controller(FloatLayout):

    current_account = ObjectProperty(allownone=True)
    current_account_string = StringProperty(allownone=True)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        # disables the roll screen until `preload_account_utils` is done
        # disabling doesn't seem to work within the scheduled method
        # self.roll_screen.toggle_widgets(False)
        self.disabled = True
        Clock.schedule_once(self._after_init)
        self._account_passwords = {}
        self._pyetheroll = None

    def _after_init(self, dt):
        """Inits pyethapp and binds events."""
        Clock.schedule_once(self.preload_account_utils)
        self.bind_roll_button()
        self.bind_current_account_string()
        self.bind_chances_roll_under()
        self.bind_wager_property()
        self.bind_profit_property()
        self.bind_screen_manager_on_current_screen()
        self.bind_keyboard()
        self.register_screens()

    def on_keyboard(self, window, key, *args):
        """
        Handles the back button (Android) and ESC key. Goes back to the
        previous screen, dicards dialogs or exits the application if none left.
        """
        if key == 27:
            if Dialog.dialogs:
                Dialog.dismiss_all_dialogs()
                return True
            from etherollapp.etheroll.ui_utils import SubScreen
            current_screen = self.screen_manager.current_screen
            # if is sub-screen loads previous and stops the propagation
            # otherwise propagates the key to exit
            if isinstance(current_screen, SubScreen):
                current_screen.on_back()
                return True
        return False

    def on_current_account(self, instance, current_account):
        self.current_account_string = (
            current_account and f'0x{current_account.address.hex()}')

    @property
    def pyetheroll(self):
        """
        Gets or creates the Etheroll object.
        Also recreates the object if the chain_id changed.
        """
        from pyetheroll.etheroll import Etheroll
        chain_id = Settings.get_stored_network()
        if self._pyetheroll is None or self._pyetheroll.chain_id != chain_id:
            self._pyetheroll = Etheroll(API_KEY_PATH, chain_id)
        return self._pyetheroll

    @property
    def account_utils(self):
        """Gets or creates the AccountUtils object so it loads lazily."""
        from etherollapp.ethereum_utils import AccountUtils
        keystore_dir = Settings.get_keystore_path()
        return AccountUtils.get_or_create(keystore_dir)

    def preload_account_utils(self, dt):
        """Preloads `AccountUtils`, since it takes few seconds on Android."""
        account_utils = self.account_utils
        self.disabled = False
        # not using that returned value, but it peaces linter
        return account_utils

    def bind_wager_property(self):
        """Binds wager recap label."""
        roll_under_recap = self.roll_screen.ids.roll_under_recap_id
        bet_size = self.roll_screen.ids.bet_size_id
        bet_size_input = bet_size.ids.bet_size_input_id
        # TODO: input validation, if `bet_size.text == ''`
        bet_size_input.bind(text=roll_under_recap.setter('wager_property'))
        # synchro once now
        roll_under_recap.wager_property = bet_size_input.text

    def bind_chances_roll_under(self):
        """Binds chances of winning recap label."""
        roll_under_recap = self.roll_screen.ids.roll_under_recap_id
        # roll under recap label
        chance_of_winning = self.roll_screen.ids.chance_of_winning_id
        chances_input = chance_of_winning.ids.chances_input_id
        # TODO: input validation, if `chances_input.text == ''`
        chances_input.bind(text=roll_under_recap.setter('roll_under_property'))
        # synchronises it now
        roll_under_recap.roll_under_property = chances_input.text

    def bind_roll_button(self):
        """Binds roll screen "Roll" button to controller roll()."""
        roll_button = self.roll_screen.ids.roll_button_id
        roll_button.bind(on_release=lambda instance: self.roll())

    def bind_current_account_string(self):
        """Binds Controller.current_account -> RollScreen.current_account"""
        roll_screen = self.roll_screen
        self.bind(
            current_account_string=roll_screen.setter('current_account_string')
        )

    def bind_keyboard(self):
        """Binds keyboard keys to actions."""
        from kivy.core.window import Window
        Window.bind(on_keyboard=self.on_keyboard)

    def bind_profit_property(self):
        """Binds profit property with bet value and chances changes."""
        # chances -> profit
        chance_of_winning = self.roll_screen.ids.chance_of_winning_id
        chances_input = chance_of_winning.ids.chances_input_id
        chances_input.bind(
            text=lambda instance, value: self.update_profit_property())
        # bet value -> profit
        bet_size = self.roll_screen.ids.bet_size_id
        bet_size_input = bet_size.ids.bet_size_input_id
        bet_size_input.bind(
            text=lambda instance, value: self.update_profit_property())
        # synchro once now
        self.update_profit_property()

    def bind_screen_manager_on_current_screen(self):
        """SwitchAccountScreen.current_account -> self.current_account."""
        def on_pre_add_widget(screen_manager, screen):
            """Should only be called twice per screen instance."""
            if type(screen) is SwitchAccountScreen and \
                    not self.screen_manager.has_screen(screen.name):
                screen.bind(current_account=self.setter('current_account'))
                screen.ids.send_id.bind(
                    current_account=self.setter('current_account'))
                screen.ids.send_id.bind(on_send=self.on_send)
        self.screen_manager.bind(on_pre_add_widget=on_pre_add_widget)

    def register_screens(self):
        # lazy loading
        from etherollapp.etheroll.about import AboutScreen
        from etherollapp.etheroll.roll_results import RollResultsScreen
        screen_dicts = {
            "about_screen": AboutScreen,
            'flashqrcode': FlashQrCodeScreen,
            "roll_results_screen": RollResultsScreen,
            "settings_screen": SettingsScreen,
            "switch_account_screen": SwitchAccountScreen,
        }
        for screen_name, screen_type in screen_dicts.items():
            self.screen_manager.register_screen(screen_type, screen_name)

    def update_profit_property(self):
        house_edge = 1.0 / 100
        bet_size = self.roll_screen.ids.bet_size_id.value
        chances_win = self.roll_screen.ids.chance_of_winning_id.value
        chances_loss = 100 - chances_win
        roll_under_recap = self.roll_screen.ids.roll_under_recap_id
        roll_under_recap.profit_property = 0
        if chances_win != 0 and chances_loss != 0:
            payout = ((chances_loss / chances_win) * bet_size) + bet_size
            payout *= (1 - house_edge)
            roll_under_recap.profit_property = payout - bet_size

    @property
    def navigation(self):
        return self.ids.navigation_id

    @property
    def screen_manager(self):
        return self.ids.screen_manager_id

    @property
    def roll_screen(self):
        return self.screen_manager.get_screen('roll_screen')

    @property
    def switch_account_screen(self):
        return self.screen_manager.get_screen('switch_account_screen')

    @property
    def roll_results_screen(self):
        return self.screen_manager.get_screen('roll_results_screen')

    @property
    def settings_screen(self):
        return self.screen_manager.get_screen('settings_screen')

    @property
    def about_screen(self):
        return self.screen_manager.get_screen('about_screen')

    def prompt_password_dialog(self, account, on_password_callback):
        """Prompt the password dialog."""
        # lazy loading
        from etherollapp.etheroll.passwordform import PasswordForm
        dialog = PasswordForm.dialog(account)

        def on_unlock_clicked(instance, dialog, account, password):
            """Caches the password and call roll method again."""
            self._account_passwords[account.address.hex()] = password
            dialog.dismiss()
            on_password_callback()

        dialog.content.bind(on_unlock=on_unlock_clicked)
        dialog.open()
        return dialog

    def get_account_password(self, account, on_password_callback):
        """Retrieve cached account password or prompt dialog."""
        address = account.address.hex()
        try:
            return self._account_passwords[address]
        except KeyError:
            self.prompt_password_dialog(account, on_password_callback)

    @staticmethod
    def on_account_none():
        """Error dialog on no account selected."""
        title = "No account selected"
        body = "Please select an account before rolling"
        dialog = Dialog.create_dialog(title, body)
        dialog.open()

    @staticmethod
    @mainthread
    def dialog_roll_success(tx_hash):
        title = "Rolled successfully"
        body = "Transaction hash:\n" + tx_hash.hex()
        dialog = Dialog.create_dialog(title, body)
        dialog.open()

    @staticmethod
    @mainthread
    def dialog_transaction_success(tx_hash):
        title = "Transaction successful"
        body = "Transaction hash:\n" + tx_hash.hex()
        dialog = Dialog.create_dialog(title, body)
        dialog.open()

    @mainthread
    def dialog_roll_error(self, exception):
        """
        Shows different error message depending on the exception.
        On "MAC mismatch" (wrong password), void the cached password so the
        user can try again refs:
        https://github.com/AndreMiras/EtherollApp/issues/9
        """
        title = "Error rolling"
        body = str(exception)
        if body == 'MAC mismatch':
            title = "Wrong password"
            body = "Can't unlock wallet, wrong password."
            account = self.current_account
            self._account_passwords.pop(account.address.hex())
        dialog = Dialog.create_dialog(title, body)
        dialog.open()

    @run_in_thread
    def player_roll_dice(
            self, bet_size_eth, chances, wallet_path, password,
            gas_price_gwei):
        """
        Sending the bet to the smart contract requires signing a transaction
        which requires CPU computation to unlock the account, hence this
        is ran in a thread.
        """
        roll_screen = self.roll_screen
        try:
            Dialog.snackbar_message("Sending bet...")
            roll_screen.toggle_widgets(False)
            bet_size_wei = int(bet_size_eth * 1e18)
            gas_price_wei = int(gas_price_gwei * 1e9)
            tx_hash = self.pyetheroll.player_roll_dice(
                bet_size_wei, chances, wallet_path, password, gas_price_wei)
        except (ValueError, ConnectionError) as exception:
            roll_screen.toggle_widgets(True)
            self.dialog_roll_error(exception)
            return
        roll_screen.toggle_widgets(True)
        self.dialog_roll_success(tx_hash)

    @staticmethod
    def start_services():
        """
        Starts both roll polling service and OSC service.
        The roll polling service is getting the OSC server connection
        parameters so it can communicate to it.
        """
        app = App.get_running_app()
        osc_server, sockname = OscAppServer.get_or_create(app)
        server_address, server_port = sockname
        print(sockname)
        arguments = {
            'osc_server_address': server_address,
            'osc_server_port': server_port,
        }
        start_roll_polling_service(arguments)

    def roll(self):
        """
        Retrieves bet parameters from user input and sends it as a signed
        transaction to the smart contract.
        """
        roll_screen = self.roll_screen
        roll_input = roll_screen.get_roll_input()
        bet_size_eth = roll_input['bet_size']
        chances = roll_input['chances']
        gas_price_gwei = Settings.get_stored_gas_price()
        account = self.current_account
        if account is None:
            self.on_account_none()
            return
        wallet_path = account.path
        password = self.get_account_password(account, self.roll)
        if password is not None:
            self.player_roll_dice(
                bet_size_eth, chances, wallet_path, password, gas_price_gwei)
            # restarts roll polling service to reset the roll activity period
            self.start_services()

    def transaction(
            self, to, amount_eth, wallet_path, password, gas_price_gwei):
        """Converts input parameters for the underlying library."""
        value = int(amount_eth * 1e18)
        gas_price_wei = int(gas_price_gwei * 1e9)
        to = to_checksum_address(to)
        Dialog.snackbar_message("Sending transaction...")
        tx_hash = self.pyetheroll.transaction(
            to, value, wallet_path, password, gas_price_wei)
        self.dialog_transaction_success(tx_hash)

    def send(self, address, amount_eth):
        """Retrieves fields to complete the `transaction()` call."""
        gas_price_gwei = Settings.get_stored_gas_price()
        account = self.current_account
        if account is None:
            self.on_account_none()
            return
        wallet_path = account.path
        password = self.get_account_password(
            account, lambda: self.send(address, amount_eth))
        if password is not None:
            self.transaction(
                address, amount_eth, wallet_path, password, gas_price_gwei)

    def on_send(self, instance, address, amount_eth):
        self.send(address, amount_eth)

    def load_switch_account(self):
        """Loads the switch account screen."""
        screen_manager = self.screen_manager
        screen_manager.transition.direction = 'right'
        screen_manager.current = 'switch_account_screen'

    def load_flash_qr_code(self):
        """Loads the flash QR Code screen."""
        # loads ZBarCam only when needed
        from kivy_garden.zbarcam import ZBarCam  # noqa
        # loads the flash QR Code screen
        self.screen_manager.transition.direction = 'right'
        self.screen_manager.current = 'flashqrcode'

    def show_qr_code(self):
        """Shows address QR Code in a dialog."""
        # lazy loading
        from kivy_garden.qrcode import QRCodeWidget
        from kivy.metrics import dp
        account = self.current_account
        if not account:
            return
        address = "0x" + account.address.hex()
        title = address
        qr_code = QRCodeWidget()
        qr_code.data = address
        dialog = Dialog.create_dialog_content_helper(
                    title=title,
                    content=qr_code)
        # workaround for MDDialog container size (too small by default)
        dialog.ids.container.size_hint_y = 1
        dialog.height = dp(500)
        dialog.add_action_button(
            "OK",
            action=lambda *x: dialog.dismiss())
        dialog.open()
        return dialog

    def copy_address_clipboard(self):
        """Copies the current account address to the clipboard."""
        # lazy loading
        from kivy.core.clipboard import Clipboard
        account = self.current_account
        if not account:
            return
        address = "0x" + account.address.hex()
        Clipboard.copy(address)

    def open_address_options(self):
        """
        Loads the address options bottom sheet.
        """
        bottom_sheet = MDListBottomSheet()
        bottom_sheet.add_item(
            'Switch account',
            lambda x: self.load_switch_account(), icon='swap-horizontal')
        bottom_sheet.add_item(
            'Show QR Code',
            lambda x: self.show_qr_code(), icon='information')
        bottom_sheet.add_item(
            'Copy address',
            lambda x: self.copy_address_clipboard(), icon='content-copy')
        bottom_sheet.open()

    @staticmethod
    def on_permission_error(exception):
        title = "Permission denied"
        body = str(exception.args)
        dialog = Dialog.create_dialog(title, body)
        dialog.open()


class DebugRavenClient(object):
    """
    The DebugRavenClient should be used in debug mode, it just raises
    the exception rather than capturing it.
    """

    def captureException(self):
        raise


class EtherollApp(App):

    theme_cls = ThemeManager()

    def build(self):
        self.icon = "docs/images/icon.png"
        self.theme_cls.theme_style = 'Dark'
        self.theme_cls.primary_palette = 'Indigo'
        Controller.start_services()
        return Controller()


def main():
    # only send Android errors to Sentry
    in_debug = platform != "android"
    client = configure_sentry(in_debug)
    try:
        EtherollApp().run()
    except Exception:
        if type(client) == Client:
            Logger.info(
                'Errors will be sent to Sentry, run with "--debug" if you '
                'are a developper and want to the error in the shell.')
        client.captureException()


if __name__ == '__main__':
    main()



from kivy.clock import Clock
from kivy.metrics import dp
from kivymd.button import MDFlatButton


class AddressButton(MDFlatButton):
    """
    Overrides MDFlatButton, makes the font slightly smaller on mobile
    by using "Body1" rather than "Button" style.
    Also shorten content size using ellipsis.
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        Clock.schedule_once(lambda dt: self.setup())

    def setup(self):
        self.set_font_and_shorten()

    def set_font_and_shorten(self):
        """
        Makes the font slightly smaller on mobile
        by using "Body1" rather than "Button" style.
        Also shorten content size using ellipsis.
        """
        content = self.ids.content
        content.font_style = 'Body1'
        content.shorten = True

        def on_parent_size(instance, size):
            # see BaseRectangularButton.width definition
            button_margin = dp(32)
            parent_width = instance.width
            # TODO: the new size should be a min() of
            # parent_width and actual content size
            content.width = parent_width - button_margin
        self.parent.bind(size=on_parent_size)
        # call it once manually, refs:
        # https://github.com/AndreMiras/PyWallet/issues/74
        on_parent_size(self.parent, None)



import os
import shutil
import unittest
from tempfile import TemporaryDirectory, mkdtemp
from unittest import mock

from etherollapp.ethereum_utils import AccountUtils
from etherollapp.pyethapp_accounts import Account

PASSWORD = "password"


class TestAccountUtils(unittest.TestCase):

    def setUp(self):
        self.keystore_dir = mkdtemp()
        self.account_utils = AccountUtils(self.keystore_dir)

    def tearDown(self):
        shutil.rmtree(self.keystore_dir, ignore_errors=True)

    def test_new_account(self):
        """
        Simple account creation test case.
        1) verifies the current account list is empty
        2) creates a new account and verify we can retrieve it
        3) tries to unlock the account
        """
        # 1) verifies the current account list is empty
        self.assertEqual(self.account_utils.get_account_list(), [])
        # 2) creates a new account and verify we can retrieve it
        password = PASSWORD
        account = self.account_utils.new_account(password, iterations=1)
        self.assertEqual(len(self.account_utils.get_account_list()), 1)
        self.assertEqual(account, self.account_utils.get_account_list()[0])
        # 3) tries to unlock the account
        # it's unlocked by default after creation
        self.assertFalse(account.locked)
        # let's lock it and unlock it back
        account.lock()
        self.assertTrue(account.locked)
        account.unlock(password)
        self.assertFalse(account.locked)

    def test_get_account_list(self):
        """
        Makes sure get_account_list() loads properly accounts from file system.
        """
        password = PASSWORD
        self.assertEqual(self.account_utils.get_account_list(), [])
        account = self.account_utils.new_account(password, iterations=1)
        self.assertEqual(len(self.account_utils.get_account_list()), 1)
        account = self.account_utils.get_account_list()[0]
        self.assertIsNotNone(account.path)
        # removes the cache copy and checks again if it gets loaded
        self.account_utils._accounts = None
        self.assertEqual(len(self.account_utils.get_account_list()), 1)
        account = self.account_utils.get_account_list()[0]
        self.assertIsNotNone(account.path)

    def test_get_account_list_error(self):
        """
        get_account_list() should not cache empty account on PermissionError.
        """
        # creates a temporary account that we'll try to list
        password = PASSWORD
        account = Account.new(password, uuid=None, iterations=1)
        account.path = os.path.join(self.keystore_dir, account.address.hex())
        with open(account.path, 'w') as f:
            f.write(account.dump())
        # `listdir()` can raise a `PermissionError`
        with mock.patch('os.listdir') as mock_listdir:
            mock_listdir.side_effect = PermissionError
            with self.assertRaises(PermissionError):
                self.account_utils.get_account_list()
        # the empty account list should not be catched and loading it again
        # should show the existing account on file system
        self.assertEqual(len(self.account_utils.get_account_list()), 1)
        self.assertEqual(
            self.account_utils.get_account_list()[0].address, account.address)

    def test_get_account_list_no_dir(self):
        """
        The keystore directory should be created if it doesn't exist, refs:
        https://github.com/AndreMiras/PyWallet/issues/133
        """
        # nominal case when the directory already exists
        with TemporaryDirectory() as keystore_dir:
            self.assertTrue(os.path.isdir(keystore_dir))
            account_utils = AccountUtils(keystore_dir)
            self.assertEqual(account_utils.get_account_list(), [])
        # when the directory doesn't exist it should also be created
        self.assertFalse(os.path.isdir(keystore_dir))
        account_utils = AccountUtils(keystore_dir)
        self.assertTrue(os.path.isdir(keystore_dir))
        self.assertEqual(account_utils.get_account_list(), [])
        shutil.rmtree(keystore_dir, ignore_errors=True)

    def test_deleted_account_dir(self):
        """
        The deleted_account_dir() helper method should be working
        with and without trailing slash.
        """
        expected_deleted_keystore_dir = '/tmp/keystore-deleted'
        keystore_dirs = [
            # without trailing slash
            '/tmp/keystore',
            # with one trailing slash
            '/tmp/keystore/',
            # with two trailing slashes
            '/tmp/keystore//',
        ]
        for keystore_dir in keystore_dirs:
            self.assertEqual(
                AccountUtils.deleted_account_dir(keystore_dir),
                expected_deleted_keystore_dir)

    def test_delete_account(self):
        """
        Creates a new account and delete it.
        Then verify we can load the account from the backup/trash location.
        """
        password = PASSWORD
        account = self.account_utils.new_account(password, iterations=1)
        address = account.address
        self.assertEqual(len(self.account_utils.get_account_list()), 1)
        # deletes the account and verifies it's not loaded anymore
        self.account_utils.delete_account(account)
        self.assertEqual(len(self.account_utils.get_account_list()), 0)
        # even recreating the AccountUtils object
        self.account_utils = AccountUtils(self.keystore_dir)
        self.assertEqual(len(self.account_utils.get_account_list()), 0)
        # tries to reload it from the backup/trash location
        deleted_keystore_dir = AccountUtils.deleted_account_dir(
            self.keystore_dir)
        self.account_utils = AccountUtils(deleted_keystore_dir)
        self.assertEqual(len(self.account_utils.get_account_list()), 1)
        self.assertEqual(
            self.account_utils.get_account_list()[0].address, address)

    def test_delete_account_already_exists(self):
        """
        If the destination (backup/trash) directory where the account is moved
        already exists, it should be handled gracefully.
        This could happens if the account gets deleted, then reimported and
        deleted again, refs:
        https://github.com/AndreMiras/PyWallet/issues/88
        """
        password = PASSWORD
        account = self.account_utils.new_account(password, iterations=1)
        # creates a file in the backup/trash folder that would conflict
        # with the deleted account
        deleted_keystore_dir = AccountUtils.deleted_account_dir(
            self.keystore_dir)
        os.makedirs(deleted_keystore_dir)
        account_filename = os.path.basename(account.path)
        deleted_account_path = os.path.join(
            deleted_keystore_dir, account_filename)
        # create that file
        open(deleted_account_path, 'a').close()
        # then deletes the account and verifies it worked
        self.assertEqual(len(self.account_utils.get_account_list()), 1)
        self.account_utils.delete_account(account)
        self.assertEqual(len(self.account_utils.get_account_list()), 0)

    def test_get_or_create(self):
        """
        Checks if the singleton is handled properly.
        It should only be different when changing keystore_dir.
        """
        account_utils = AccountUtils.get_or_create(self.keystore_dir)
        assert account_utils == AccountUtils.get_or_create(self.keystore_dir)
        with TemporaryDirectory() as keystore_dir:
            assert account_utils != AccountUtils.get_or_create(keystore_dir)
            assert AccountUtils.get_or_create(keystore_dir) == \
                AccountUtils.get_or_create(keystore_dir)



from kivymd.spinner import MDSpinner

from etherollapp.etheroll.ui_utils import load_kv_from_py

load_kv_from_py(__file__)


class ScrollViewSpinder(MDSpinner):

    def toggle(self, show):
        """Actually shrinking and hidding it."""
        # by default it's in "shown" mode and doesn't have attributes saved
        if not hasattr(self, 'previous_size_hint'):
            self.previous_size_hint = self.size_hint.copy()
            self.previous_opacity = self.opacity
        elif show:
            self.size_hint = self.previous_size_hint
            self.opacity = self.previous_opacity
        else:
            self.size_hint = (0, 0)
            self.opacity = 0



import os

from etherollapp.pyethapp_accounts import Account


class AccountUtils:

    singleton = None

    def __init__(self, keystore_dir):
        self.keystore_dir = keystore_dir
        self._accounts = None
        os.makedirs(keystore_dir, exist_ok=True)

    @classmethod
    def get_or_create(cls, keystore_dir):
        """Gets or creates the AccountUtils object so it loads lazily."""
        if cls.singleton is None or \
                cls.singleton.keystore_dir != keystore_dir:
            cls.singleton = cls(keystore_dir=keystore_dir)
        return cls.singleton

    def get_account_list(self):
        """Returns the Account list."""
        if self._accounts is not None:
            return self._accounts
        keyfiles = []
        for item in os.listdir(self.keystore_dir):
            item_path = os.path.join(self.keystore_dir, item)
            if os.path.isfile(item_path):
                keyfiles.append(item_path)
        # starts caching after `listdir()` call so if it fails
        # (e.g. `PermissionError`) account list won't be empty next call
        self._accounts = []
        for keyfile in keyfiles:
            account = Account.load(path=keyfile)
            self._accounts.append(account)
        return self._accounts

    def new_account(self, password, iterations=None):
        """Creates an account on the disk and returns it."""
        account = Account.new(password, uuid=None, iterations=iterations)
        account.path = os.path.join(self.keystore_dir, account.address.hex())
        self.add_account(account)
        return account

    def add_account(self, account):
        with open(account.path, 'w') as f:
            f.write(account.dump())
        if self._accounts is None:
            self._accounts = []
        self._accounts.append(account)
        return account

    @staticmethod
    def deleted_account_dir(keystore_dir):
        """
        Given a `keystore_dir`, returns the corresponding
        `deleted_keystore_dir`.
        >>> keystore_dir = '/tmp/keystore'
        >>> AccountUtils.deleted_account_dir(keystore_dir)
        u'/tmp/keystore-deleted'
        >>> keystore_dir = '/tmp/keystore/'
        >>> AccountUtils.deleted_account_dir(keystore_dir)
        u'/tmp/keystore-deleted'
        """
        keystore_dir = keystore_dir.rstrip('/')
        keystore_dir_name = os.path.basename(keystore_dir)
        deleted_keystore_dir_name = "%s-deleted" % (keystore_dir_name)
        deleted_keystore_dir = os.path.join(
            os.path.dirname(keystore_dir),
            deleted_keystore_dir_name)
        return deleted_keystore_dir

    def delete_account(self, account):
        """
        Deletes the given `account` from the `keystore_dir` directory.
        Then deletes it from the `AccountsService` account manager instance.
        In fact, moves it to another location; another directory at the same
        level.
        """
        # lazy loading
        import shutil
        keystore_dir = self.keystore_dir
        deleted_keystore_dir = self.deleted_account_dir(keystore_dir)
        # create the deleted account dir if required
        if not os.path.exists(deleted_keystore_dir):
            os.makedirs(deleted_keystore_dir)
        # "removes" it from the file system
        account_filename = os.path.basename(account.path)
        deleted_account_path = os.path.join(
            deleted_keystore_dir, account_filename)
        shutil.move(account.path, deleted_account_path)
        self._accounts.remove(account)



from kivy.app import App
from kivy.clock import mainthread
from kivy.properties import StringProperty
from kivy.uix.boxlayout import BoxLayout

from etherollapp.etheroll.ui_utils import Dialog, load_kv_from_py
from etherollapp.etheroll.utils import run_in_thread

load_kv_from_py(__file__)


class CreateNewAccount(BoxLayout):
    """Makes it possible to create json keyfiles."""

    new_password1 = StringProperty()
    new_password2 = StringProperty()

    def verify_password_field(self):
        """Makes sure passwords are matching and are not void."""
        passwords_matching = self.new_password1 == self.new_password2
        passwords_not_void = self.new_password1 != ''
        return passwords_matching and passwords_not_void

    def verify_fields(self):
        """Verifies password fields are valid."""
        return self.verify_password_field()

    @staticmethod
    def try_unlock(account, password):
        """
        Just as a security measure, verifies we can unlock
        the newly created account with provided password.
        """
        # making sure it's locked first
        account.lock()
        try:
            account.unlock(password)
        except ValueError:
            title = "Unlock error"
            body = ""
            body += "Couldn't unlock your account.\n"
            body += "The issue should be reported."
            dialog = Dialog.create_dialog(title, body)
            dialog.open()
            return

    @mainthread
    def on_account_created(self, account):
        """Switches to the newly created account. Clears the form."""
        # TODO:
        # it would be better design if only the Controller new about the
        # CreateNewAccount and not the opposite
        controller = App.get_running_app().root
        controller.current_account = account
        self.new_password1 = ''
        self.new_password2 = ''

    @mainthread
    def toggle_widgets(self, enabled):
        """Enables/disables account creation widgets."""
        self.disabled = not enabled

    @mainthread
    def show_redirect_dialog(self):
        title = "Account created, redirecting..."
        body = ""
        body += "Your account was created, "
        body += "you will be redirected to the overview."
        dialog = Dialog.create_dialog(title, body)
        dialog.open()

    def load_landing_page(self):
        """Returns to the landing page."""
        controller = App.get_running_app().root
        screen_manager = controller.screen_manager
        screen_manager.transition.direction = 'right'
        screen_manager.current = 'roll_screen'

    @run_in_thread
    def create_account(self):
        """
        Creates an account from provided form.
        Verify we can unlock it.
        Disables widgets during the process, so the user doesn't try
        to create another account during the process.
        """
        self.toggle_widgets(False)
        if not self.verify_fields():
            Dialog.show_invalid_form_dialog()
            self.toggle_widgets(True)
            return
        password = self.new_password1
        Dialog.snackbar_message("Creating account...")
        controller = App.get_running_app().root
        account = controller.account_utils.new_account(password=password)
        Dialog.snackbar_message("Created!")
        self.toggle_widgets(True)
        self.on_account_created(account)
        # CreateNewAccount.try_unlock(account, password)
        self.show_redirect_dialog()
        self.load_landing_page()
        return account



import os
import shutil
import threading
import time
import unittest
from functools import partial
from tempfile import mkdtemp
from unittest import mock
from unittest.mock import patch

from hexbytes import HexBytes
from kivy.clock import Clock
from kivy.core.image import Image
from requests.exceptions import ConnectionError

from etherollapp.etheroll.constants import BASE_DIR
from etherollapp.etheroll.controller import EtherollApp
from etherollapp.etheroll.ui_utils import Dialog
from etherollapp.tests.utils import PyEtherollTestUtils


def get_camera_class():
    """
    Continuous integration providers don't have a camera available.
    """
    if os.environ.get('CI', False):
        Camera = None
    else:
        from kivy.core.camera import Camera
    return Camera


def patch_core_camera():
    Camera = get_camera_class()
    return patch('kivy.uix.camera.CoreCamera', wraps=Camera)


def patch_fetch_update_balance():
    return patch('etherollapp.etheroll.roll.RollScreen.fetch_update_balance')


def advance_frames(count):
    """
    Borrowed from Kivy 1.10.0+ /kivy/tests/common.py
    GraphicUnitTest.advance_frames()
    Makes it possible to to wait for UI to process, refs #110.
    """
    from kivy.base import EventLoop
    for i in range(count):
        EventLoop.idle()


def advance_frames_for_screen():
    """Gives screen switch animation time to render."""
    advance_frames(50)


def advance_frames_for_drawer():
    """Gives drawer switch animation time to render."""
    advance_frames_for_screen()


def load_roll_screen(screen_manager):
    """Loads back the default screen."""
    with patch_fetch_update_balance() as m_fetch_update_balance:
        # loads back the default screen
        screen_manager.current = 'roll_screen'
    assert m_fetch_update_balance.call_args_list == [mock.call()]
    advance_frames_for_screen()


class UITestCase(unittest.TestCase):

    def setUp(self):
        """
        Sets a temporay KEYSTORE_PATH, so keystore directory and related
        application files will be stored here until tearDown().
        """
        self.temp_path = mkdtemp()
        os.environ['KEYSTORE_PATH'] = self.temp_path
        # see `kivy.App._get_user_data_dir()`
        os.environ['XDG_CONFIG_HOME'] = self.temp_path

    def tearDown(self):
        shutil.rmtree(self.temp_path, ignore_errors=True)

    def helper_setup(self, app):
        pass
        # etheroll.SCREEN_SWITCH_DELAY = 0.001

    # sleep function that catches `dt` from Clock
    def pause(*args):
        time.sleep(0.000001)

    @staticmethod
    def join_threads():
        """Joins pending threads except for main and OSC threads."""
        threads = threading.enumerate()
        # lists all threads but the main one
        for thread in threads[2:]:
            thread.join()

    @staticmethod
    def wait_mock_called(m, timeout=1):
        """Returns True if `m` was called before the `timeout` in seconds."""
        step = 0.1
        while timeout > 0 and not m.called:
            time.sleep(step)
            timeout -= step
        return m.called

    def helper_test_empty_account(self, app):
        """Verifies the UI behaves as expected on empty account list."""
        controller = app.root
        accounts = controller.account_utils.get_account_list()
        # loading the app with empty account directory
        self.assertEqual(len(accounts), 0)
        # retrieving the roll button, to click it
        roll_screen = controller.roll_screen
        roll_button = roll_screen.ids.roll_button_id
        self.assertEqual(roll_button.text, 'Roll')
        roll_button.dispatch('on_release')
        # it should open the error dialog
        dialogs = Dialog.dialogs
        self.assertEqual(len(dialogs), 1)
        dialog = dialogs[0]
        self.assertEqual(dialog.title, 'No account selected')
        dialog.dismiss()
        self.assertEqual(len(dialogs), 0)

    def helper_test_toolbar(self, app):
        """Opens the about screen using the toolbar."""
        controller = app.root
        toolbar = controller.ids.toolbar_id
        left_action_items = toolbar.left_action_items
        # checking action items displayed, it should be only the menu
        self.assertEqual(len(left_action_items), 1)
        menu_action_item = left_action_items[0]
        self.assertEqual(menu_action_item[0], 'menu')
        self.assertEqual(
            menu_action_item[1].__qualname__,
            'CustomToolbar.load_default_buttons.<locals>.<lambda>')
        # this is reflected in the toolbar left_actions widget
        left_actions = toolbar.ids.left_actions
        self.assertEqual(len(left_actions.children), 1)
        # clicking the menu action item should load the navigation drawer
        left_actions.children[0].dispatch('on_release')
        advance_frames_for_drawer()
        # checking the drawer items
        navigation = controller.ids.navigation_id
        navigation_drawer = navigation.ids.navigation_drawer_id
        navigation_drawer_items = navigation_drawer.ids.list.children
        self.assertEqual(
            [child.text for child in navigation_drawer_items],
            ['About', 'Settings', 'Wallet', 'Roll results', 'Home'])
        # clicking the about one
        about_item = navigation_drawer_items[0]
        about_item.dispatch('on_release')
        advance_frames_for_drawer()
        self.assertEqual(
            controller.screen_manager.current, 'about_screen')
        # toolbar should now be loaded with the back button
        left_action_items = toolbar.left_action_items
        self.assertEqual(len(left_action_items), 1)
        menu_action_item = left_action_items[0]
        self.assertEqual(menu_action_item[0], 'arrow-left')
        # going back to main screen
        left_actions = toolbar.ids.left_actions
        with patch_fetch_update_balance() as m_fetch_update_balance:
            left_actions.children[0].dispatch('on_release')
        assert m_fetch_update_balance.call_args_list == [mock.call()]
        advance_frames_for_screen()
        self.assertEqual(
            controller.screen_manager.current, 'roll_screen')

    def helper_test_about_screen(self, app):
        """Verifies the about screen loads and shows infos."""
        controller = app.root
        screen_manager = controller.screen_manager
        # verify the landing screen is loaded by default
        screen = screen_manager.children[0]
        self.assertEqual(screen.name, 'roll_screen')
        # loads the about and verify
        screen_manager.current = 'about_screen'
        advance_frames_for_screen()
        screen = screen_manager.children[0]
        self.assertEqual(screen.name, 'about_screen')
        # checks about screen content
        about_screen_manager = screen.children[0].children[1]
        about_overview = about_screen_manager.get_screen('about_overview')
        about_content = about_overview.children[0].children[0].text
        self.assertTrue('EtherollApp version: ' in about_content)
        self.assertTrue(
            'https://github.com/AndreMiras/EtherollApp' in about_content)
        load_roll_screen(screen_manager)

    def helper_no_account_checks(self, app):
        """
        Helper method for checking widget are in expected state on no account.
        """
        controller = app.root
        screen_manager = controller.screen_manager
        account_utils = controller.account_utils
        # makes sure no account are loaded
        self.assertEqual(len(account_utils.get_account_list()), 0)
        # loads the switch account screen
        screen_manager.current = 'switch_account_screen'
        advance_frames_for_screen()
        switch_account_screen = controller.switch_account_screen
        advance_frames_for_screen()
        # it should open the warning dialog
        dialogs = Dialog.dialogs
        self.assertEqual(len(dialogs), 1)
        dialog = dialogs[0]
        self.assertEqual(dialog.title, 'No account found')
        dialog.dismiss()
        self.assertEqual(len(dialogs), 0)
        # loads the create new account tab
        create_new_account_nav_item = \
            switch_account_screen.ids.create_new_account_nav_item_id
        create_new_account_nav_item.dispatch('on_tab_press')
        # verifies no current account is setup
        self.assertEqual(switch_account_screen.current_account, None)

    def helper_test_create_first_account(self, app):
        """Creates the first account."""
        controller = app.root
        account_utils = controller.account_utils
        screen_manager = controller.screen_manager
        screen_manager.current = 'switch_account_screen'
        advance_frames_for_screen()
        # makes sure it still complains for no account
        dialogs = Dialog.dialogs
        self.assertEqual(len(dialogs), 1)
        dialog = dialogs[0]
        self.assertEqual(dialog.title, 'No account found')
        dialog.dismiss()
        switch_account_screen = controller.switch_account_screen
        # retrieves the create_new_account widget
        create_new_account = switch_account_screen.ids.create_new_account_id
        # retrieves widgets (password fields, sliders and buttons)
        new_password1_id = create_new_account.ids.new_password1_id
        new_password2_id = create_new_account.ids.new_password2_id
        create_account_button_id = \
            create_new_account.ids.create_account_button_id
        # fills them up with same password
        new_password1_id.text = new_password2_id.text = "password"
        # before clicking the create account button,
        # only the main and OSC threads are running
        threads = threading.enumerate()
        self.assertEqual(len(threads), 2, threads[-1]._target)
        main_thread = threads[0]
        self.assertEqual(type(main_thread), threading._MainThread)
        with patch_fetch_update_balance() as m_fetch_update_balance:
            # click the create account button
            create_account_button_id.dispatch('on_release')
            # after submitting the account creation thread should also run
            threads = threading.enumerate()
            self.assertEqual(len(threads), 3)
            self.assertEqual(type(main_thread), threading._MainThread)
            create_account_thread = threads[2]
            self.assertTrue(
                'function CreateNewAccount.create_account'
                in str(create_account_thread._target))
            # waits for the end of the thread
            create_account_thread.join()
        assert m_fetch_update_balance.call_args_list == [mock.call()]
        # thread has ended, main & OSC threads only are running again
        threads = threading.enumerate()
        self.assertEqual(len(threads), 2)
        main_thread = threads[0]
        self.assertEqual(type(main_thread), threading._MainThread)
        # verifies the account was created
        self.assertEqual(len(account_utils.get_account_list()), 1)
        # TODO verify the form fields were voided
        # self.assertEqual(new_password1_id.text, '')
        # self.assertEqual(new_password2_id.text, '')
        # we should get redirected to the overview page
        advance_frames_for_screen()
        self.assertEqual(controller.screen_manager.current, 'roll_screen')
        # the new account should be loaded in the controller
        self.assertEqual(
            controller.current_account,
            account_utils.get_account_list()[0])
        self.join_threads()
        # check the redirect dialog
        dialogs = Dialog.dialogs
        self.assertEqual(len(dialogs), 1)
        dialog = dialogs[0]
        self.assertEqual(dialog.title, 'Account created, redirecting...')
        dialog.dismiss()
        self.assertEqual(len(dialogs), 0)

    def helper_test_create_account_form(self, app):
        """
        Create account form validation checks.
        Testing both not matching and empty passwords.
        """
        controller = app.root
        screen_manager = controller.screen_manager
        screen_manager.current = 'switch_account_screen'
        advance_frames_for_screen()
        switch_account_screen = controller.switch_account_screen
        account_utils = controller.account_utils
        # number of existing accounts before the test
        account_count_before = account_utils.get_account_list()
        # retrieves the create_new_account widget
        create_new_account = switch_account_screen.ids.create_new_account_id
        # retrieves widgets (password fields, sliders and buttons)
        new_password1_id = create_new_account.ids.new_password1_id
        new_password2_id = create_new_account.ids.new_password2_id
        create_account_button_id = \
            create_new_account.ids.create_account_button_id
        passwords_to_try = [
            # passwords not matching
            {
                'new_password1': 'not matching1',
                'new_password2': 'not matching2'
            },
            # passwords empty
            {
                'new_password1': '',
                'new_password2': ''
            },
        ]
        for password_dict in passwords_to_try:
            new_password1_id.text = password_dict['new_password1']
            new_password2_id.text = password_dict['new_password2']
            # makes the account creation fast
            # before clicking the create account button,
            # only the main and OSC threads are running
            self.assertEqual(len(threading.enumerate()), 2)
            main_thread = threading.enumerate()[0]
            self.assertEqual(type(main_thread), threading._MainThread)
            # click the create account button
            create_account_button_id.dispatch('on_release')
            # after submitting the account verification thread should run
            threads = threading.enumerate()
            # since we may run into race condition with threading.enumerate()
            # we make the test conditional
            if len(threads) == 3:
                create_account_thread = threading.enumerate()[2]
                self.assertEqual(
                    type(create_account_thread), threading.Thread)
                self.assertTrue(
                    'function CreateNewAccount.create_account'
                    in str(create_account_thread._target))
                # waits for the end of the thread
                create_account_thread.join()
            # the form should popup an error dialog
            dialogs = Dialog.dialogs
            self.assertEqual(len(dialogs), 1)
            dialog = dialogs[0]
            self.assertEqual(dialog.title, 'Invalid form')
            dialog.dismiss()
            self.assertEqual(len(dialogs), 0)
            # no account were created
            self.assertEqual(
                len(account_count_before),
                len(account_utils.get_account_list()))

    def helper_test_chances_input_binding(self, app):
        """Makes sure the chances input works as expected, refs #46."""
        controller = app.root
        # retrieves both widgets from roll screen
        roll_screen = controller.roll_screen
        chance_of_winning = roll_screen.ids.chance_of_winning_id
        chances_slider = chance_of_winning.ids.chances_slider_id
        chances_input = chance_of_winning.ids.chances_input_id
        # makes sure both values are the same
        self.assertEqual(str(chances_slider.value), chances_input.text)
        # checks the default value
        self.assertEqual(chances_slider.value, 50)
        # verifies values are binded input -> slider
        chances_input.text = '30'
        chances_input.dispatch('on_text_validate')
        self.assertEqual(chances_slider.value, 30)
        # slider -> input
        chances_slider.value = 70
        self.assertEqual(chances_input.text, '70')

    def helper_test_roll_history(self, app):
        """Roll history screen should display recent rolls, refs #61."""
        bet_results_logs = PyEtherollTestUtils.bet_results_logs
        bet_logs = PyEtherollTestUtils.bet_logs
        merged_logs = [
            {'bet_log': bet_logs[0], 'bet_result': bet_results_logs[0]},
            {'bet_log': bet_logs[0], 'bet_result': bet_results_logs[0]},
            # not yet resolved (no `LogResult`)
            {'bet_log': bet_logs[2], 'bet_result': None},
        ]
        controller = app.root
        # makes sure an account is selected
        self.assertIsNotNone(controller.current_account)
        screen_manager = controller.screen_manager
        # patches library with fake recent rolls
        with patch('pyetheroll.etheroll.Etheroll.get_merged_logs') \
                as m_get_merged_logs:
            m_get_merged_logs.return_value = merged_logs
            screen_manager.current = 'roll_results_screen'
            # waits and makes sure the mock to be called, refs #138
            self.assertTrue(self.wait_mock_called(m_get_merged_logs))
            # since get_merged_logs got called, the get_last_results thread
            # should be over, only main & OSC threads only are running
            thread_info = [(t, t._target) for t in threading.enumerate()]
            self.assertEqual(
                len(threading.enumerate()), 2, thread_info)
            advance_frames_for_screen()
            # thread has ended, main & OSC threads only are running again
            self.assertEqual(len(threading.enumerate()), 2)
            main_thread = threading.enumerate()[0]
            self.assertEqual(type(main_thread), threading._MainThread)
        # verifies recent rolls appear
        roll_results_screen = controller.roll_results_screen
        roll_list = roll_results_screen.ids.roll_list_id
        # roll results items should be displayed
        items = roll_list.children
        self.assertEqual(len(items), len(merged_logs))
        load_roll_screen(screen_manager)

    def helper_test_roll_history_no_tx(self, app):
        """
        When going to the roll history screen with an account selected that has
        no transaction history, the application should fail gracefully.
        """
        # TODO: currently disabled because of a py-etherscan-api bug, refs:
        # https://github.com/AndreMiras/EtherollApp/issues/67
        return
        controller = app.root
        # makes sure an account is selected
        self.assertIsNotNone(controller.current_account)
        screen_manager = controller.screen_manager
        screen_manager.current = 'roll_results_screen'
        # make sure the application is not complaining
        dialogs = Dialog.dialogs
        self.assertEqual(len(dialogs), 1)
        dialog = dialogs[0]
        self.assertEqual(dialog.title, 'No transaction found')
        load_roll_screen(screen_manager)

    def helper_test_roll_history_no_account(self, app):
        """
        When going to the roll history screen with no account selected,
        the application should complain and bring back to the main screen.
        """
        controller = app.root
        screen_manager = controller.screen_manager
        controller.current_account = None
        screen_manager = controller.screen_manager
        with patch_fetch_update_balance() as m_fetch_update_balance:
            screen_manager.current = 'roll_results_screen'
            advance_frames_for_screen()
        assert m_fetch_update_balance.call_args_list == [mock.call()]
        dialogs = Dialog.dialogs
        self.assertEqual(len(dialogs), 1)
        dialog = dialogs[0]
        self.assertEqual(dialog.title, 'No account selected')
        dialog.dismiss()
        # we should get redirected to the overview page
        advance_frames_for_screen()
        self.assertEqual(controller.screen_manager.current, 'roll_screen')
        # e.g. fetch_update_balance thread
        self.join_threads()

    def helper_test_roll(self, app):
        """Trying to place a valid roll."""
        controller = app.root
        # makes sure an account is selected
        # TODO: do it the proper way by clicking the UI
        controller.current_account = \
            controller.account_utils.get_account_list()[0]
        # retrieving the roll button, to click it
        roll_screen = controller.roll_screen
        roll_button = roll_screen.ids.roll_button_id
        self.assertEqual(roll_button.text, 'Roll')
        roll_button.dispatch('on_release')
        # it should open the error dialog
        dialogs = Dialog.dialogs
        self.assertEqual(len(dialogs), 1)
        dialog = dialogs[0]
        self.assertEqual(dialog.title, 'Enter your password')
        dialog.content.password = 'password'
        unlock_button = dialog._action_buttons[0]
        with patch('pyetheroll.etheroll.Etheroll.player_roll_dice') \
                as m_player_roll_dice:
            m_player_roll_dice.return_value = HexBytes(
                '0x7be6e37621eb12db7dc535954345f69'
                'd8cc5644b2de0ec32a344ca33c3054237')
            unlock_button.dispatch('on_release')
            threads = threading.enumerate()
            # since we may run into race condition with threading.enumerate()
            # we make the test conditional
            if len(threads) == 3:
                # rolls should be fetched from a thread
                self.assertEqual(len(threads), 3)
                player_roll_dice_thread = threads[2]
                self.assertEqual(
                    type(player_roll_dice_thread), threading.Thread)
                self.assertTrue(
                    'function Controller.player_roll_dice'
                    in str(player_roll_dice_thread._target))
                # waits for the end of the thread
                player_roll_dice_thread.join()
        advance_frames_for_screen()
        # thread has ended, main & OSC threads only are running again
        self.assertEqual(len(threading.enumerate()), 2)
        main_thread = threading.enumerate()[0]
        self.assertEqual(type(main_thread), threading._MainThread)
        # a confirmation dialog with transaction hash should pop
        dialogs = Dialog.dialogs
        self.assertEqual(len(dialogs), 1)
        dialog = dialogs[0]
        self.assertEqual(dialog.title, 'Rolled successfully')
        dialog.dismiss()
        self.assertEqual(len(dialogs), 0)

    def helper_test_roll_connection_error(self, app):
        """
        Makes sure ConnectionError on roll is handled gracefully, refs #111.
        """
        controller = app.root
        # retrieving the roll button, to click it
        roll_screen = controller.roll_screen
        roll_button = roll_screen.ids.roll_button_id
        self.assertEqual(roll_button.text, 'Roll')
        with patch('web3.eth.Eth.getTransactionCount') \
                as m_getTransactionCount:
            m_getTransactionCount.side_effect = \
                ConnectionError('Whatever ConnectionError')
            roll_button.dispatch('on_release')
            threads = threading.enumerate()
            # since we may run into race condition with threading.enumerate()
            # we make the test conditional
            if len(threads) == 3:
                # rolls should be fetched from a thread
                self.assertEqual(len(threads), 3)
                player_roll_dice_thread = threads[2]
                self.assertEqual(
                    type(player_roll_dice_thread), threading.Thread)
                self.assertTrue(
                    'function Controller.player_roll_dice'
                    in str(player_roll_dice_thread._target))
                # waits for the end of the thread
                player_roll_dice_thread.join()
        advance_frames_for_screen()
        # thread has ended, main & OSC threads only are running again
        self.assertEqual(len(threading.enumerate()), 2)
        main_thread = threading.enumerate()[0]
        self.assertEqual(type(main_thread), threading._MainThread)
        # an error dialog should pop
        dialogs = Dialog.dialogs
        self.assertEqual(len(dialogs), 1)
        dialog = dialogs[0]
        self.assertEqual(dialog.title, 'Error rolling')
        self.assertEqual(dialog.content.text, 'Whatever ConnectionError')
        dialog.dismiss()
        self.assertEqual(len(dialogs), 0)

    def helper_test_roll_password(self, app):
        """
        Makes sure wrong passwords are handled properly.
        Relevant error messages should prompt and it should be possible to
        try again, refs #9.
        """
        controller = app.root
        # makes sure an account is selected
        self.assertIsNotNone(controller.current_account)
        # voids cached passwords
        controller._account_passwords = {}
        # retrieving the roll button, to click it
        roll_screen = controller.roll_screen
        roll_button = roll_screen.ids.roll_button_id
        self.assertEqual(roll_button.text, 'Roll')
        roll_button.dispatch('on_release')
        # it should open the error dialog
        dialogs = Dialog.dialogs
        self.assertEqual(len(dialogs), 1)
        dialog = dialogs[0]
        self.assertEqual(dialog.title, 'Enter your password')
        dialog.content.password = 'wrong password'
        unlock_button = dialog._action_buttons[0]
        unlock_button.dispatch('on_release')
        # runs in a thread since password unlocking and tx signing takes time
        threads = threading.enumerate()
        # since we may run into race condition with threading.enumerate()
        # we make the test conditional
        if len(threads) == 3:
            # rolls should be fetched from a thread
            self.assertEqual(len(threads), 3)
            player_roll_dice_thread = threads[2]
            self.assertEqual(
                type(player_roll_dice_thread), threading.Thread)
            self.assertTrue(
                'function Controller.player_roll_dice'
                in str(player_roll_dice_thread._target))
            # waits for the end of the thread
            player_roll_dice_thread.join()
        advance_frames_for_screen()
        # thread has ended, main & OSC threads only are running again
        self.assertEqual(len(threading.enumerate()), 2)
        main_thread = threading.enumerate()[0]
        self.assertEqual(type(main_thread), threading._MainThread)
        # an error dialog should pop
        dialogs = Dialog.dialogs
        self.assertEqual(len(dialogs), 1)
        dialog = dialogs[0]
        self.assertEqual(dialog.title, 'Wrong password')
        dialog.dismiss()
        # the wrong password should not get cached and a password form should
        # be prompted again next time we try to roll
        roll_button.dispatch('on_release')
        # it should open the error dialog
        dialogs = Dialog.dialogs
        self.assertEqual(len(dialogs), 1)
        dialog = dialogs[0]
        self.assertEqual(dialog.title, 'Enter your password')
        dialog.dismiss()

    def helper_test_settings_screen(self, app):
        """Verifies the settings screen loads and shows infos."""
        controller = app.root
        screen_manager = controller.screen_manager
        # verify the landing screen is loaded by default
        screen = screen_manager.children[0]
        self.assertEqual(screen.name, 'roll_screen')
        # loads the about and verify
        screen_manager.current = 'settings_screen'
        advance_frames_for_screen()
        screen = screen_manager.children[0]
        self.assertEqual(screen.name, 'settings_screen')
        # checks settings screen attributes
        self.assertEqual(screen.is_ui_mainnet(), True)
        self.assertEqual(screen.is_ui_testnet(), False)
        load_roll_screen(screen_manager)

    def helper_test_transaction(self, app):
        """Trying to send ETH."""
        controller = app.root
        switch_account_screen = controller.switch_account_screen
        send_subscreen = switch_account_screen.ids.send_id
        send_subscreen.parent.dispatch('on_tab_press')
        advance_frames_for_screen()
        # makes sure an account is selected
        controller.current_account = \
            controller.account_utils.get_account_list()[0]
        # retrieving the send button, to click it
        send_to_address = '0x46044beAa1E985C67767E04dE58181de5DAAA00F'
        send_subscreen.ids.send_to_id.text = send_to_address
        amount_eth = 1.2
        send_subscreen.ids.amount_eth_id.text = f'{amount_eth}'
        send_button = send_subscreen.ids.send_button_id
        self.assertEqual(send_button.text, 'Send')
        send_button.dispatch('on_release')
        # it should open the password dialog
        dialogs = Dialog.dialogs
        self.assertEqual(len(dialogs), 1)
        dialog = dialogs[0]
        self.assertEqual(dialog.title, 'Enter your password')
        password = 'password'
        dialog.content.password = password
        unlock_button = dialog._action_buttons[0]
        with patch('pyetheroll.etheroll.Etheroll.transaction', autospec=True) \
                as m_transaction:
            m_transaction.return_value = HexBytes(
                '0x7be6e37621eb12db7dc535954345f69'
                'd8cc5644b2de0ec32a344ca33c3054237')
            unlock_button.dispatch('on_release')
        pyetheroll = controller.pyetheroll
        to = send_to_address
        value = int(amount_eth * 1e18)
        wallet_path = controller.current_account.path
        gas_price_wei = mock.ANY
        self.assertEqual(m_transaction.mock_calls, [
            mock.call(
                pyetheroll, to, value, wallet_path, password, gas_price_wei)
        ])
        advance_frames_for_screen()
        threads = threading.enumerate()
        # thread has ended, main & OSC threads only are running again
        self.assertEqual(len(threads), 2, threads[-1]._target)
        main_thread = threads[0]
        self.assertEqual(type(main_thread), threading._MainThread)
        # a confirmation dialog with transaction hash should pop
        dialogs = Dialog.dialogs
        self.assertEqual(len(dialogs), 1)
        dialog = dialogs[0]
        self.assertEqual(dialog.title, 'Transaction successful')
        dialog.dismiss()
        self.assertEqual(len(dialogs), 0)
        # screen_manager = controller.screen_manager
        # load_roll_screen(screen_manager)

    def helper_test_flashqrcode(self, app):
        """Verifies the flash QRCode screen loads and can flash codes."""
        controller = app.root
        with patch_core_camera():
            controller.load_flash_qr_code()
        advance_frames_for_screen()
        screen_manager = controller.screen_manager
        self.assertEqual(screen_manager.current, 'flashqrcode')
        flashqrcode_screen = screen_manager.get_screen('flashqrcode')
        zbarcam = flashqrcode_screen.ids.zbarcam_id
        fixture_path = os.path.join(BASE_DIR, 'tests', 'address_qrcode.png')
        texture = Image(fixture_path).texture
        camera = mock.Mock(texture=texture)
        zbarcam._on_texture(camera)
        switch_account_screen = controller.switch_account_screen
        send_screen = switch_account_screen.ids.send_id
        self.assertEqual(
            send_screen.ids.send_to_id.text,
            '0x46044beaa1e985c67767e04de58181de5daaa00f')
        advance_frames_for_screen()

    # main test function
    def run_test(self, app, *args):
        Clock.schedule_interval(self.pause, 0.000001)
        self.helper_setup(app)
        # lets it finish to init
        advance_frames(1)
        self.helper_test_empty_account(app)
        self.helper_test_toolbar(app)
        self.helper_test_about_screen(app)
        self.helper_test_create_first_account(app)
        self.helper_test_create_account_form(app)
        self.helper_test_chances_input_binding(app)
        self.helper_test_roll_history(app)
        self.helper_test_roll_history_no_tx(app)
        self.helper_test_roll_history_no_account(app)
        self.helper_test_roll(app)
        self.helper_test_roll_connection_error(app)
        self.helper_test_roll_password(app)
        self.helper_test_settings_screen(app)
        self.helper_test_transaction(app)
        self.helper_test_flashqrcode(app)
        # Comment out if you are editing the test, it'll leave the
        # Window opened.
        app.stop()

    # same named function as the filename(!)
    def test_ui_base(self):
        app = EtherollApp()
        p = partial(self.run_test, app)
        Clock.schedule_once(p, 0.000001)
        app.run()


if __name__ == '__main__':
    unittest.main()



"""
The OSC app client connects to the OSC app server to communicate with the
application from another process.
OscAppClient -> OscAppServer -> App
"""
import argparse

from oscpy.client import OSCClient


class OscAppClient:
    """OSC client that talks to the OscAppServer to update the app process."""

    def __init__(self, address, port):
        self.address = address
        self.port = port
        self.osc = OSCClient(address, port)

    def send_ping(self):
        for i in range(10):
            self.osc.send_message(b'/ping', [i])

    def send_refresh_balance(self):
        self.osc.send_message(b'/refresh_balance', [])


def main():
    """Test main that sends a ping message to the specified server."""
    parser = argparse.ArgumentParser()
    parser.add_argument("--host",  default='localhost')
    parser.add_argument("--port", type=int, default=8000)
    args = parser.parse_args()
    osc_app_client = OscAppClient(args.host, args.port)
    osc_app_client.send_ping()


if __name__ == '__main__':
    main()



import unittest

from etherollapp.tests import test_ethereum_utils, test_import


def suite():
    modules = [
        test_ethereum_utils, test_import,
    ]
    test_suite = unittest.TestSuite()
    for module in modules:
        test_suite.addTest(unittest.TestLoader().loadTestsFromModule(module))
    return test_suite



import json
import os

import eth_account
from eth_keyfile import create_keyfile_json, decode_keyfile_json
from eth_keys import keys
from eth_utils import decode_hex, encode_hex, remove_0x_prefix


def to_string(value) -> bytes:
    if isinstance(value, bytes):
        return value
    elif isinstance(value, str):
        return bytes(value, 'utf-8')
    elif isinstance(value, int):
        return bytes(str(value), 'utf-8')
    else:
        raise ValueError('Cannot convert to string')


class Account:
    """
    Represents an account.
    :ivar keystore: the key store as a dictionary (as decoded from json)
    :ivar locked: `True` if the account is locked and neither private nor
    public keys can be accessed, otherwise `False`
    :ivar path: absolute path to the associated keystore file (`None` for
    in-memory accounts)
    """

    def __init__(self, keystore: dict, password: bytes = None, path=None):
        self._address = None
        self.keystore = keystore
        if 'address' in self.keystore:
            self._address = decode_hex(self.keystore['address'])
        self.locked = True
        if password is not None:
            password = to_string(password)
            self.unlock(password)
        if path is not None:
            self.path = os.path.abspath(path)
        else:
            self.path = None

    @classmethod
    def new(cls, password: bytes, key: bytes = None, uuid=None, path=None,
            iterations=None):
        """
        Create a new account.
        Note that this creates the account in memory and does not store it on
        disk.
        :param password: the password used to encrypt the private key
        :param key: the private key, or `None` to generate a random one
        :param uuid: an optional id
        """
        if key is None:
            account = eth_account.Account.create()
            key = account.key

        password = to_string(password)

        # encrypted = eth_account.Account.encrypt(account.key, password)
        keystore = create_keyfile_json(key, password, iterations=iterations)
        keystore['id'] = uuid
        return Account(keystore, password, path)

    @classmethod
    def load(cls, path, password: bytes = None):
        """
        Load an account from a keystore file.
        :param path: full path to the keyfile
        :param password: the password to decrypt the key file or `None` to
        leave it encrypted
        """
        with open(path) as f:
            keystore = json.load(f)
        # if not keys.check_keystore_json(keystore):
        #     raise ValueError('Invalid keystore file')
        return Account(keystore, password, path=path)

    def dump(self, include_address=True, include_id=True):
        """
        Dump the keystore for later disk storage.
        The result inherits the entries `'crypto'` and `'version`' from
        `account.keystore`, and adds `'address'` and `'id'` in accordance with
        the parameters `'include_address'` and `'include_id`'.
        If address or id are not known, they are not added, even if requested.
        :param include_address: flag denoting if the address should be included
        or not
        :param include_id: flag denoting if the id should be included or not
        """
        d = {}
        d['crypto'] = self.keystore['crypto']
        d['version'] = self.keystore['version']
        if include_address and self.address is not None:
            d['address'] = encode_hex(self.address)
        if include_id and self.uuid is not None:
            d['id'] = str(self.uuid)
        return json.dumps(d)

    def unlock(self, password: bytes):
        """
        Unlock the account with a password.
        If the account is already unlocked, nothing happens, even if the
        password is wrong.
        :raises: :exc:`ValueError` (originating in ethereum.keys) if the
        password is wrong (and the account is locked)
        """
        if self.locked:
            password = to_string(password)
            self._privkey = decode_keyfile_json(self.keystore, password)
            self.locked = False
            # get address such that it stays accessible after a subsequent lock
            self.address

    def lock(self):
        """
        Relock an unlocked account.
        This method sets `account.privkey` to `None` (unlike `account.address`
        which is preserved).
        After calling this method, both `account.privkey` and `account.pubkey`
        are `None.
        `account.address` stays unchanged, even if it has been derived from the
        private key.
        """
        self._privkey = None
        self.locked = True

    @property
    def privkey(self):
        """The account's private key or `None` if the account is locked."""
        if not self.locked:
            return self._privkey
        else:
            return None

    @property
    def pubkey(self):
        """The account's public key or `None` if the account is locked."""
        if not self.locked:
            pk = keys.PrivateKey(self.privkey)
            return remove_0x_prefix(pk.public_key.to_address())
        else:
            return None

    @property
    def address(self):
        """
        The account's address or `None` if the address is not stored in the key
        file and cannot be reconstructed (because the account is locked).
        """
        if self._address:
            pass
        elif 'address' in self.keystore:
            self._address = decode_hex(self.keystore['address'])
        elif not self.locked:
            pk = keys.PrivateKey(self.privkey)
            self._address = decode_hex(pk.public_key.to_address())
        else:
            return None
        return self._address

    @property
    def uuid(self):
        """
        An optional unique identifier, formatted according to UUID version 4,
        or `None` if the account does not have an id.
        """
        try:
            return self.keystore['id']
        except KeyError:
            return None

    @uuid.setter
    def uuid(self, value):
        """Set the UUID. Set it to `None` in order to remove it."""
        if value is not None:
            self.keystore['id'] = value
        elif 'id' in self.keystore:
            self.keystore.pop('id')

    # TODO: not yet migrated
    # def sign_tx(self, tx):
    #     """
    #     Sign a Transaction with the private key of this account.
    #     If the account is unlocked, this is equivalent to
    #     `tx.sign(account.privkey)`.
    #     :param tx: the :class:`ethereum.transactions.Transaction` to sign
    #     :raises: :exc:`ValueError` if the account is locked
    #     """
    #     if self.privkey:
    #         tx.sign(self.privkey)
    #     else:
    #         raise ValueError('Locked account cannot sign tx')

    def __repr__(self):
        if self.address is not None:
            address = encode_hex(self.address)
        else:
            address = '?'
        return f'<Account(address={address}, id={self.uuid})>'



from kivy.logger import LOG_LEVELS
from kivy.utils import platform
from raven import Client
from raven.conf import setup_logging
from raven.handlers.logging import SentryHandler

from etherollapp.version import __version__


def configure_sentry(in_debug=False):
    """
    Configure the Raven client, or create a dummy one if `in_debug` is `True`.
    """
    key = 'b290ecc8934f4cb599e6fa6af6cc5cc2'
    # the public DSN URL is not available on the Python client
    # so we're exposing the secret and will be revoking it on abuse
    # https://github.com/getsentry/raven-python/issues/569
    secret = '0ae02bcb5a75467d9b4431042bb98cb9'
    project_id = '1111738'
    dsn = 'https://{key}:{secret}@sentry.io/{project_id}'.format(
        key=key, secret=secret, project_id=project_id)
    if in_debug:
        client = DebugRavenClient()
    else:
        client = Client(dsn=dsn, release=__version__)
        # adds context for Android devices
        if platform == 'android':
            from jnius import autoclass
            Build = autoclass("android.os.Build")
            VERSION = autoclass('android.os.Build$VERSION')
            android_os_build = {
                'model': Build.MODEL,
                'brand': Build.BRAND,
                'device': Build.DEVICE,
                'manufacturer': Build.MANUFACTURER,
                'version_release': VERSION.RELEASE,
            }
            client.user_context({'android_os_build': android_os_build})
        # Logger.error() to Sentry
        # https://docs.sentry.io/clients/python/integrations/logging/
        handler = SentryHandler(client)
        handler.setLevel(LOG_LEVELS.get('error'))
        setup_logging(handler)
    return client


class DebugRavenClient(object):
    """
    The DebugRavenClient should be used in debug mode, it just raises
    the exception rather than capturing it.
    """

    def captureException(self):
        raise



from kivy.app import App
from kivy.clock import Clock
from kivymd.toolbar import Toolbar

from etherollapp.etheroll.ui_utils import load_kv_from_py

load_kv_from_py(__file__)


class CustomToolbar(Toolbar):
    """Toolbar with helper method for loading default/back buttons."""

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        Clock.schedule_once(self.load_default_buttons)

    def load_default_buttons(self, dt=None):
        app = App.get_running_app()
        self.left_action_items = [
            ['menu', lambda x: app.root.navigation.toggle_nav_drawer()]]
        self.right_action_items = [[
                'dots-vertical',
                lambda x: app.root.navigation.toggle_nav_drawer()]]

    def load_back_button(self, function):
        self.left_action_items = [['arrow-left', lambda x: function()]]



from kivy.app import App
from kivy.clock import Clock, mainthread
from kivy.properties import ObjectProperty
from kivy.uix.boxlayout import BoxLayout
from kivymd.list import OneLineListItem

from etherollapp.etheroll.ui_utils import Dialog, SubScreen, load_kv_from_py
from etherollapp.etheroll.utils import run_in_thread

load_kv_from_py(__file__)


class SwitchAccount(BoxLayout):
    __events__ = ('on_account_selected',)

    def on_release(self, list_item):
        """Fires on_account_selected() event."""
        self.dispatch('on_account_selected', list_item.account)

    def on_account_selected(self, *args):
        """Default handler."""
        pass

    def create_item(self, account):
        """Creates an account list item from given account."""
        address = "0x" + account.address.hex()
        list_item = OneLineListItem(text=address)
        # makes sure the address doesn't overlap on small screen
        list_item.ids._lbl_primary.shorten = True
        list_item.account = account
        list_item.bind(on_release=lambda x: self.on_release(x))
        return list_item

    @mainthread
    def update_account_list(self, accounts):
        account_list_id = self.ids.account_list_id
        account_list_id.clear_widgets()
        if len(accounts) == 0:
            self.on_empty_account_list()
        for account in accounts:
            list_item = self.create_item(account)
            account_list_id.add_widget(list_item)

    @run_in_thread
    def load_account_list(self):
        """
        Fills account list widget from library account list.
        This is running in a thread because first call to `account_utils` will
        initialize it lazily which takes few seconds on Android devices.
        """
        self.controller = App.get_running_app().root
        self.toggle_spinner(show=True)
        accounts = []
        accounts = self.controller.account_utils.get_account_list()
        self.update_account_list(accounts)
        self.toggle_spinner(show=False)

    @staticmethod
    def on_empty_account_list():
        controller = App.get_running_app().root
        keystore_dir = controller.account_utils.keystore_dir
        title = "No account found"
        body = f"No account found in:\n{keystore_dir}"
        dialog = Dialog.create_dialog(title, body)
        dialog.open()

    def toggle_spinner(self, show):
        spinner = self.ids.spinner_id
        spinner.toggle(show)


class SwitchAccountScreen(SubScreen):
    current_account = ObjectProperty(allownone=True)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        Clock.schedule_once(self._after_init)

    def _after_init(self, dt):
        """Binds SwitchAccount.on_account_selected() event."""
        switch_account = self.ids.switch_account_id
        switch_account.bind(
            on_account_selected=lambda
            instance, account: self.on_account_selected(account))

    def on_account_selected(self, account):
        """Sets current account and loads previous screen."""
        self.current_account = account
        self.on_back()



from datetime import datetime


class PyEtherollTestUtils:
    """
    Copy of some helper members from:
    https://github.com/AndreMiras/pyetheroll/blob/20181031/
    tests/test_etheroll.py#L18
    """
    bet_logs = [
        {
            'bet_id': (
                '15e007148ec621d996c886de0f2b88a0'
                '3af083aa819e851a51133dc17b6e0e5b'),
            'bet_value_ether': 0.45,
            'datetime': datetime(2018, 4, 7, 0, 17, 6),
            'profit_value_ether': 44.1,
            'reward_value_ether': 44.55,
            'roll_under': 2,
            'timestamp': '0x5ac80e02',
            'transaction_hash': (
                '0xf363906a9278c4dd300c50a3c9a2790'
                '0bb85df60596c49f7833c232f2944d1cb')
        },
        {
            'bet_id': (
                '14bae6b4711bdc5e3db19983307a9208'
                '1e2e7c1d45161117bdf7b8b509d1abbe'),
            'bet_value_ether': 0.45,
            'datetime': datetime(2018, 4, 7, 0, 20, 14),
            'profit_value_ether': 6.97,
            'reward_value_ether': 7.42,
            'roll_under': 7,
            'timestamp': '0x5ac80ebe',
            'transaction_hash': (
                '0x0df8789552248edf1dd9d06a7a90726'
                'f1bc83a9c39f315b04efb6128f0d02146')
        },
        {
            # that one would not have been yet resolved (no `LogResult`)
            'bet_id': (
                'c2997a1bad35841b2c30ca95eea9cb08'
                'c7b101bc14d5aa8b1b8a0facea793e05'),
            'bet_value_ether': 0.5,
            'datetime': datetime(2018, 4, 7, 0, 23, 46),
            'profit_value_ether': 3.31,
            'reward_value_ether': 3.81,
            'roll_under': 14,
            'timestamp': '0x5ac80f92',
            'transaction_hash': (
                '0x0440f1013a5eafd88f16be6b5612b6e'
                '051a4eb1b0b91a160c680295e7fab5bfe')
        }
    ]

    bet_results_logs = [
        {
            'bet_id': (
                '15e007148ec621d996c886de0f2b88a0'
                '3af083aa819e851a51133dc17b6e0e5b'),
            'bet_value_ether': 0.45,
            'datetime': datetime(2018, 4, 7, 0, 17, 55),
            'dice_result': 86,
            'roll_under': 2,
            'timestamp': '0x5ac80e33',
            'transaction_hash': (
                '0x3505de688dc20748eb5f6b3efd6e6d3'
                '66ea7f0737b4ab17035c6b60ab4329f2a')
        },
        {
            'bet_id': (
                '14bae6b4711bdc5e3db19983307a9208'
                '1e2e7c1d45161117bdf7b8b509d1abbe'),
            'bet_value_ether': 0.45,
            'datetime': datetime(2018, 4, 7, 0, 20, 54),
            'dice_result': 51,
            'roll_under': 7,
            'timestamp': '0x5ac80ee6',
            'transaction_hash': (
                '0x42df3e3136957bcc64226206ed177d5'
                '7ac9c31e116290c8778c97474226d3092')
        },
    ]



import threading
from io import StringIO

from kivy.utils import platform


def run_in_thread(fn):
    """
    Decorator to run a function in a thread.
    >>> 1 + 1
    2
    >>> @run_in_thread
    ... def threaded_sleep(seconds):
    ...     from time import sleep
    ...     sleep(seconds)
    >>> thread = threaded_sleep(0.1)
    >>> type(thread)
    <class 'threading.Thread'>
    >>> thread.is_alive()
    True
    >>> thread.join()
    >>> thread.is_alive()
    False
    """
    def run(*k, **kw):
        t = threading.Thread(target=fn, args=k, kwargs=kw)
        t.start()
        return t
    return run


def check_write_permission():
    """Android runtime storage permission check."""
    if platform != "android":
        return True
    from android.permissions import Permission, check_permission
    permission = Permission.WRITE_EXTERNAL_STORAGE
    return check_permission(permission)


def check_request_write_permission():
    """Android runtime storage permission check & request."""
    had_permission = check_write_permission()
    if not had_permission:
        from android.permissions import Permission, request_permission
        permission = Permission.WRITE_EXTERNAL_STORAGE
        request_permission(permission)
    return had_permission


class StringIOCBWrite(StringIO):
    """Inherits StringIO, provides callback on write."""

    def __init__(self, initial_value='', newline='\n', callback_write=None):
        """
        Overloads the StringIO.__init__() makes it possible to hook a callback
        for write operations.
        """
        self.callback_write = callback_write
        super().__init__(initial_value, newline)

    def write(self, s):
        """
        Calls the StringIO.write() method then the callback_write with
        given string parameter.
        """
        super().write(s)
        if self.callback_write is not None:
            self.callback_write(s)



import json

from kivy.utils import platform


def start_roll_polling_service(arguments=None):
    """
    Starts the roll polling service.
    If the service is already running, it won't be started twice.
    """
    if platform != 'android':
        return
    from jnius import autoclass
    package_name = 'etheroll'
    package_domain = 'com.github.andremiras'
    service_name = 'service'
    service_class = '{}.{}.Service{}'.format(
        package_domain, package_name, service_name.title())
    service = autoclass(service_class)
    mActivity = autoclass('org.kivy.android.PythonActivity').mActivity
    argument = json.dumps(arguments)
    service.start(mActivity, argument)



"""
The OSC app server makes it possible to communicate with the application from
another process.
OscAppServer -> App
"""
from time import sleep

from oscpy.server import OSCThreadServer, ServerClass

osc = OSCThreadServer()


@ServerClass
class OscAppServer:
    """OSC server used to update the app process."""

    _osc_server = None

    def __init__(self, app=None):
        """app: instance of the application to talk to"""
        self.app = app

    @classmethod
    def get_or_create(cls, app=None):
        """
        Creates the OSC server and binds the app to it.
        app: instance of the application to talk to
        """
        if cls._osc_server is None:
            osc.listen(default=True)
            cls._osc_server = cls(app)
        sockname = osc.getaddress()
        return cls._osc_server, sockname

    @classmethod
    def stop(cls):
        osc.stop()
        cls._osc_server = None

    @osc.address_method(b'/ping')
    def _callback_ping(self, *args):
        """Test method that will reply with a pong to the sender."""
        print(f'OscAppServer.ping(): {args}')
        osc.answer(b'/pong')

    @osc.address_method(b'/pong')
    def _callback_pong(self, *args):
        """Test pong method."""
        print(f'OscAppServer.pong(): {args}')

    @osc.address_method(b'/refresh_balance')
    def _callback_refresh_balance(self, *args):
        """Refreshes roll screen balance."""
        print(f'OscAppServer.refresh_balance(): {args}')
        controller = self.app.root
        roll_screen = controller.roll_screen
        roll_screen.fetch_update_balance()


def main():
    """Test main for running the OSC app server."""
    osc_app_server, sockname = OscAppServer.get_or_create()
    print(sockname)
    sleep(100)


if __name__ == '__main__':
    main()



import binascii
import tempfile
import unittest
from unittest import mock

from kivy.app import App
from pyetheroll.constants import ChainID

from etherollapp.service.main import EtherollApp, MonitorRollsService


def patch_platform():
    return mock.patch('etherollapp.service.main.platform', 'android')


def patch_jnius(m_jnius=mock.MagicMock()):
    return mock.patch.dict('sys.modules', jnius=m_jnius)


def patch_get_abi():
    return_value = (
        '[{"constant":true,"inputs":[],"name":"minBet","outputs":[{"na'
        'me":"","type":"uint256"}],"payable":false,"stateMutability":"'
        'view","type":"function"}]')
    return mock.patch(
        'etherscan.contracts.Contract.get_abi', return_value=return_value)


class TestEtherollApp(unittest.TestCase):
    """Unit tests EtherollApp methods."""

    def test_init(self):
        """
        Checks the EtherollApp initializes correctly.
        Checks the singleton via App.get_running_app() is available.
        """
        App._running_app = None
        assert App.get_running_app() is None
        app = EtherollApp()
        assert App.get_running_app() == app
        assert app.name == 'etheroll'

    def test_get_user_data_dir(self):
        """Verifies `_get_user_data_dir()` returns the app directory."""
        app = EtherollApp()
        assert app._get_user_data_dir().endswith('.config/etheroll')

    def test_get_user_data_dir_android(self):
        """On Android, pyjnius is used to call getAbsolutePath()."""
        app = EtherollApp()
        m_jnius = mock.MagicMock()
        with patch_platform(), patch_jnius(m_jnius):
            assert app._get_user_data_dir()
        assert m_jnius.cast().getAbsolutePath.call_args_list == [mock.call()]


class TestMonitorRollsService(unittest.TestCase):
    """Unit tests MonitorRollsService methods."""

    def patch_get_merged_logs(m_get):
        return mock.patch('etherollapp.service.main.Etheroll.get_merged_logs')

    def test_init(self):
        """
        Initializes MonitorRollsService with/without osc server port param.
        """
        service = MonitorRollsService()
        assert service is not None
        assert service.osc_app_client is None
        osc_server_port = 1234
        service = MonitorRollsService(osc_server_port)
        assert service.osc_app_client is not None

    def test_run(self):
        """
        Verifies the `run()` method exits calling `set_auto_restart_service()`.
        """
        service = MonitorRollsService()
        with mock.patch(
                'etherollapp.service.main.NO_ROLL_ACTIVITY_PERDIOD_SECONDS',
                0), mock.patch.object(
                    MonitorRollsService, 'set_auto_restart_service'
                ) as m_set_auto_restart_service:
            service.run()
        assert m_set_auto_restart_service.call_args_list == [mock.call(False)]

    def test_account_utils(self):
        """
        Makes sures accessing the AccountUtils object is possible even when the
        running app singleton wasn't accessible prior to creating the service.
        """
        App._running_app = None
        assert App.get_running_app() is None
        service = MonitorRollsService()
        assert App.get_running_app() is not None
        assert service.account_utils is not None

    def test_set_auto_restart_service(self):
        """
        On Android it should be calling `setAutoRestartService()`. On other OS
        this is a NOP.
        """
        service = MonitorRollsService()
        m_jnius = mock.MagicMock()
        with patch_jnius(m_jnius):
            service.set_auto_restart_service()
        assert m_jnius.autoclass.called is False
        with patch_platform(), patch_jnius(m_jnius):
            service.set_auto_restart_service()
        assert m_jnius.autoclass.called is True
        mservice = m_jnius.autoclass().mService
        assert mservice.setAutoRestartService.call_args_list == [
            mock.call(True)
        ]

    def test_pyetheroll(self):
        """
        Checks that pyetheroll is cached.
        The cached value should be updated on (testnet/mainnet) network change.
        """
        service = MonitorRollsService()
        assert service._pyetheroll is None
        with tempfile.TemporaryDirectory() as temp_path, \
                patch_get_abi() as m_get_abi, \
                mock.patch.dict('os.environ', {'XDG_CONFIG_HOME': temp_path}):
            assert service.pyetheroll is not None
            assert m_get_abi.mock_calls == [mock.call()]
            assert service._pyetheroll is not None
            pyetheroll = service.pyetheroll
            # it's obviously pointing to the same object for now,
            # but shouldn't not be later after we update some settings
            assert pyetheroll == service.pyetheroll
            assert pyetheroll.chain_id == ChainID.MAINNET
        # the cached pyetheroll object is invalidated if the network changes
        with mock.patch(
                'etherollapp.service.main.Settings.get_stored_network'
                ) as m_get_stored_network, patch_get_abi() as m_get_abi:
            m_get_stored_network.return_value = ChainID.ROPSTEN
            assert service.pyetheroll.chain_id == ChainID.ROPSTEN
            assert service.pyetheroll != pyetheroll
        assert m_get_abi.mock_calls == [mock.call()]

    def test_pull_account_rolls(self):
        """
        Makes sure the merged logs are fetched and cached.
        Also checks were're notifying on merged logs changes.
        """
        service = MonitorRollsService()
        address = '46044beAa1E985C67767E04dE58181de5DAAA00F'
        m_account = mock.MagicMock()
        m_account.address = binascii.unhexlify(address)
        # beforehand the `MonitorRollsService.merged_logs` is not yet cached
        assert service.merged_logs == {}
        assert service.last_roll_activity is None
        merged_logs = []
        with self.patch_get_merged_logs() as m_get_merged_logs, \
                patch_get_abi() as m_get_abi:
            m_get_merged_logs.return_value = merged_logs
            service.pull_account_rolls(m_account)
        assert m_get_merged_logs.mock_calls == [
            mock.call(address=f'0x{address.lower()}')
        ]
        assert m_get_abi.mock_calls == [mock.call()]
        # then the `merged_logs` for this address should be cached
        assert service.merged_logs == {f'0x{address.lower()}': merged_logs}
        assert service.last_roll_activity is None
        # if the `merged_logs` differs, we consider it was a roll activity
        merged_logs = ['something else']
        with self.patch_get_merged_logs() as m_get_merged_logs, \
                mock.patch.object(
                    MonitorRollsService, 'do_notify') as m_do_notify:
            m_get_merged_logs.return_value = merged_logs
            service.pull_account_rolls(m_account)
        assert m_get_merged_logs.mock_calls == [
            mock.call(address=f'0x{address.lower()}')
        ]
        assert m_do_notify.call_args_list == [mock.call(merged_logs)]

    def test_pull_accounts_rolls(self):
        """
        Checks `pull_accounts_rolls()` iterates over the accounts and call
        `pull_account_rolls()` for each.
        """
        service = MonitorRollsService()
        address = '46044beAa1E985C67767E04dE58181de5DAAA00F'
        m_account = mock.MagicMock()
        m_account.address = binascii.unhexlify(address)
        with mock.patch(
                'etherollapp.service.main.AccountUtils.get_account_list'
                ) as m_get_account_list, \
                mock.patch.object(
                    MonitorRollsService, 'pull_account_rolls'
                ) as m_pull_account_rolls:
            m_get_account_list.return_value = [m_account]
            service.pull_accounts_rolls()
        assert m_pull_account_rolls.call_args_list == [
            mock.call(m_account),
        ]

    @unittest.skip("Not implemented")
    def test_do_notify(self):
        """Not yet implemented."""
        raise NotImplementedError


if __name__ == '__main__':
    unittest.main()



import os

from kivy.app import App
from kivy.utils import platform
from pyetheroll.constants import DEFAULT_GAS_PRICE_GWEI, ChainID

from etherollapp.etheroll.constants import KEYSTORE_DIR_SUFFIX
from etherollapp.etheroll.store import Store

NETWORK_SETTINGS = 'network'
GAS_PRICE_SETTINGS = 'gas_price'
PERSIST_KEYSTORE_SETTINGS = 'persist_keystore'


class Settings:
    """Screen for configuring network, gas price..."""

    @classmethod
    def get_stored_network(cls):
        """Retrieves last stored network value, defaults to Mainnet."""
        store = Store.get_store()
        try:
            network_dict = store[NETWORK_SETTINGS]
        except KeyError:
            network_dict = {}
        network_name = network_dict.get(
            'value', ChainID.MAINNET.name)
        network = ChainID[network_name]
        return network

    @classmethod
    def set_stored_network(cls, network: ChainID):
        """Persists network settings."""
        store = Store.get_store()
        store.put(NETWORK_SETTINGS, value=network.name)

    @classmethod
    def is_stored_mainnet(cls):
        network = cls.get_stored_network()
        return network == ChainID.MAINNET

    @classmethod
    def is_stored_testnet(cls):
        network = cls.get_stored_network()
        return network == ChainID.ROPSTEN

    @classmethod
    def get_stored_gas_price(cls):
        """
        Retrieves stored gas price value, defaults to DEFAULT_GAS_PRICE_GWEI.
        """
        store = Store.get_store()
        try:
            gas_price_dict = store[GAS_PRICE_SETTINGS]
        except KeyError:
            gas_price_dict = {}
        gas_price = gas_price_dict.get(
            'value', DEFAULT_GAS_PRICE_GWEI)
        return gas_price

    @classmethod
    def set_stored_gas_price(cls, gas_price: int):
        """Persists gas price settings."""
        store = Store.get_store()
        store.put(GAS_PRICE_SETTINGS, value=gas_price)

    @classmethod
    def is_persistent_keystore(cls):
        """
        Retrieves the settings value regarding the keystore persistency.
        Defaults to False.
        """
        store = Store.get_store()
        try:
            persist_keystore_dict = store[PERSIST_KEYSTORE_SETTINGS]
        except KeyError:
            persist_keystore_dict = {}
        persist_keystore = persist_keystore_dict.get(
            'value', False)
        return persist_keystore

    @classmethod
    def set_is_persistent_keystore(cls, persist_keystore: bool):
        """Saves keystore persistency settings."""
        store = Store.get_store()
        store.put(PERSIST_KEYSTORE_SETTINGS, value=persist_keystore)

    @staticmethod
    def get_persistent_keystore_path():
        app = App.get_running_app()
        # TODO: hardcoded path, refs:
        # https://github.com/AndreMiras/EtherollApp/issues/145
        return os.path.join('/sdcard', app.name)

    @staticmethod
    def get_non_persistent_keystore_path():
        app = App.get_running_app()
        return app.user_data_dir

    @classmethod
    def _get_android_keystore_prefix(cls):
        """
        Returns the Android keystore path prefix.
        The location differs based on the persistency user settings.
        """
        if cls.is_persistent_keystore():
            keystore_dir_prefix = cls.get_persistent_keystore_path()
        else:
            keystore_dir_prefix = cls.get_non_persistent_keystore_path()
        return keystore_dir_prefix

    @classmethod
    def get_keystore_path(cls):
        """
        Returns the keystore directory path.
        This can be overriden by the `KEYSTORE_PATH` environment variable.
        """
        keystore_path = os.environ.get('KEYSTORE_PATH')
        if keystore_path is not None:
            return keystore_path
        KEYSTORE_DIR_PREFIX = os.path.expanduser("~")
        if platform == "android":
            KEYSTORE_DIR_PREFIX = cls._get_android_keystore_prefix()
        keystore_path = os.path.join(
            KEYSTORE_DIR_PREFIX, KEYSTORE_DIR_SUFFIX)
        return keystore_path



import os
import threading
from typing import List

from kivy.app import App
from kivy.clock import mainthread
from kivy.lang import Builder
from kivy.metrics import dp
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.screenmanager import Screen
from kivymd.dialog import MDDialog
from kivymd.label import MDLabel
from kivymd.snackbar import Snackbar
from layoutmargin import AddMargin, MarginLayout


def load_kv_from_py(f):
    """Loads file.kv for given file.py."""
    filename = os.path.basename(os.path.splitext(f)[0])
    Builder.load_file(
        os.path.join(
            os.path.dirname(os.path.abspath(f)),
            filename + '.kv'
        )
    )


class Dialog:

    # keeps track of all dialogs alive
    dialogs: List[MDDialog] = []
    __lock = threading.Lock()

    @staticmethod
    @mainthread
    def snackbar_message(text):
        Snackbar(text=text).show()

    @classmethod
    def show_invalid_form_dialog(cls):
        title = "Invalid form"
        body = "Please check form fields."
        dialog = cls.create_dialog(title, body)
        dialog.open()

    @classmethod
    def on_dialog_dismiss(cls, dialog):
        """Removes it from the dialogs track list."""
        with cls.__lock:
            try:
                cls.dialogs.remove(dialog)
            except ValueError:
                # fails silently if the dialog was dismissed twice, refs:
                # https://github.com/AndreMiras/PyWallet/issues/89
                pass

    @classmethod
    def dismiss_all_dialogs(cls):
        """Dispatches dismiss event for all dialogs."""
        # keeps a local copy since we're altering them as we iterate
        dialogs = cls.dialogs[:]
        for dialog in dialogs:
            dialog.dismiss()

    @classmethod
    def create_dialog_content_helper(cls, title, content):
        """
        Creates a dialog from given title and content.
        Adds it to the dialogs track list.
        """
        # TODO
        dialog = MDDialog(
                        title=title,
                        content=content,
                        size_hint=(.8, None),
                        height=dp(250),
                        auto_dismiss=False)
        dialog.bind(on_dismiss=cls.on_dialog_dismiss)
        with cls.__lock:
            cls.dialogs.append(dialog)
        return dialog

    @classmethod
    def create_dialog_helper(cls, title, body):
        """
        Creates a dialog from given title and body.
        Adds it to the dialogs track list.
        """
        content = MDLabel(
                    font_style='Body1',
                    theme_text_color='Secondary',
                    text=body,
                    size_hint_y=None,
                    valign='top')
        content.bind(texture_size=content.setter('size'))
        dialog = cls.create_dialog_content_helper(title, content)
        return dialog

    @classmethod
    def create_dialog(cls, title, body):
        """
        Creates a dialog from given title and body.
        Adds it to the dialogs track list.
        Appends dismiss action.
        """
        dialog = cls.create_dialog_helper(title, body)
        dialog.add_action_button(
                "Dismiss",
                action=lambda *x: dialog.dismiss())
        return dialog


class SubScreen(Screen):
    """Helper parent class for updating toolbar on enter/leave."""

    def on_back(self):
        self.manager.transition.direction = 'right'
        self.manager.current = 'roll_screen'

    def on_enter(self):
        """Loads the toolbar back button."""
        app = App.get_running_app()
        app.root.ids.toolbar_id.load_back_button(self.on_back)

    def on_leave(self):
        """Loads the toolbar default button."""
        app = App.get_running_app()
        app.root.ids.toolbar_id.load_default_buttons()


class BoxLayoutMarginLayout(MarginLayout, BoxLayout):
    pass


class BoxLayoutAddMargin(AddMargin, BoxLayout):
    pass



import os

BASE_DIR = os.path.dirname(os.path.dirname(__file__))
# default pyethapp keystore path
KEYSTORE_DIR_SUFFIX = ".config/pyethapp/keystore/"
API_KEY_PATH = os.path.join(BASE_DIR, 'api_key.json')
NO_ACCOUNT_SELECTED_STRING = 'No account selected'



"""
Adapted version of:
https://github.com/ethereum/pyethapp/blob/7fdec62/
pyethapp/tests/test_accounts.py
"""
import json
import unittest
from builtins import str
from uuid import uuid4

from eth_keys import keys
from eth_utils import remove_0x_prefix
from past.utils import old_div

from etherollapp.pyethapp_accounts import Account


class TestAccountUtils(unittest.TestCase):

    privkey = None
    password = None
    uuid = None
    account = None
    keystore = None

    @classmethod
    def setUpClass(cls):
        cls.privkey = bytes.fromhex(
            'e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855')
        cls.password = 'secret'
        cls.uuid = str(uuid4())
        # keystore generation takes a while, so make this module scoped
        cls.account = Account.new(
            cls.password, cls.privkey, cls.uuid, iterations=1)
        # `account.keystore` might not contain address and id
        cls.keystore = json.loads(cls.account.dump())

    def test_account_creation(self):
        account = self.account
        privkey = self.privkey
        uuid = self.uuid
        assert not account.locked
        assert account.privkey == privkey
        pk = keys.PrivateKey(privkey)
        assert account.address.hex() == remove_0x_prefix(
            pk.public_key.to_address())
        assert account.uuid == uuid

    def test_locked(self):
        keystore = self.keystore
        uuid = self.uuid
        account = Account(keystore)
        assert account.locked
        assert account.address.hex() == remove_0x_prefix(keystore['address'])
        assert account.privkey is None
        assert account.pubkey is None
        assert account.uuid == uuid
        keystore2 = keystore.copy()
        keystore2.pop('address')
        account = Account(keystore2)
        assert account.locked
        assert account.address is None
        assert account.privkey is None
        assert account.pubkey is None
        assert account.uuid == uuid

    def test_unlock(self):
        keystore = self.keystore
        password = self.password
        privkey = self.privkey
        account = Account(keystore)
        assert account.locked
        account.unlock(password)
        assert not account.locked
        assert account.privkey == privkey
        pk = keys.PrivateKey(privkey)
        assert account.address.hex() == remove_0x_prefix(
            pk.public_key.to_address())

    def test_unlock_wrong(self):
        keystore = self.keystore
        password = self.password
        account = Account(keystore)
        assert account.locked
        with self.assertRaises(ValueError):
            account.unlock(password + '1234')
        assert account.locked
        with self.assertRaises(ValueError):
            account.unlock('4321' + password)
        assert account.locked
        with self.assertRaises(ValueError):
            account.unlock(password[:old_div(len(password), 2)])
        assert account.locked
        account.unlock(password)
        assert not account.locked
        account.unlock(password + 'asdf')
        assert not account.locked
        account.unlock(password + '1234')
        assert not account.locked

    def test_lock(self):
        account = self.account
        password = self.password
        privkey = self.privkey
        assert not account.locked
        pk = keys.PrivateKey(privkey)
        assert account.address.hex() == remove_0x_prefix(
            pk.public_key.to_address())
        assert account.privkey == privkey
        assert account.pubkey is not None
        account.unlock(password + 'fdsa')
        account.lock()
        assert account.locked
        pk = keys.PrivateKey(privkey)
        assert account.address.hex() == remove_0x_prefix(
            pk.public_key.to_address())
        assert account.privkey is None
        assert account.pubkey is None
        with self.assertRaises(ValueError):
            account.unlock(password + 'fdsa')
        account.unlock(password)

    def test_address(self):
        keystore = self.keystore
        password = self.password
        privkey = self.privkey
        keystore_wo_address = keystore.copy()
        keystore_wo_address.pop('address')
        account = Account(keystore_wo_address)
        assert account.address is None
        account.unlock(password)
        account.lock()
        pk = keys.PrivateKey(privkey)
        assert account.address.hex() == remove_0x_prefix(
            pk.public_key.to_address())

    def test_dump(self):
        account = self.account
        keystore = json.loads(
            account.dump(include_address=True, include_id=True))
        required_keys = set(['crypto', 'version'])
        assert set(keystore.keys()) == required_keys | set(['address', 'id'])
        assert remove_0x_prefix(keystore['address']) == account.address.hex()
        assert keystore['id'] == account.uuid
        keystore = json.loads(
            account.dump(include_address=False, include_id=True))
        assert set(keystore.keys()) == required_keys | set(['id'])
        assert keystore['id'] == account.uuid
        keystore = json.loads(
            account.dump(include_address=True, include_id=False))
        assert set(keystore.keys()) == required_keys | set(['address'])
        assert remove_0x_prefix(keystore['address']) == account.address.hex()
        keystore = json.loads(
            account.dump(include_address=False, include_id=False))
        assert set(keystore.keys()) == required_keys

    def test_uuid_setting(self):
        account = self.account
        uuid = account.uuid
        account.uuid = 'asdf'
        assert account.uuid == 'asdf'
        account.uuid = None
        assert account.uuid is None
        assert 'id' not in account.keystore
        account.uuid = uuid
        assert account.uuid == uuid
        assert account.keystore['id'] == uuid

    # TODO: not yet migrated
    # def test_sign(account, password):
    #     from ethereum.transactions import Transaction
    #     tx = Transaction(1, 0, 10**6, account.address, 0, '')
    #     account.sign_tx(tx)
    #     assert tx.sender == account.address
    #     account.lock()
    #     with pytest.raises(ValueError):
    #         account.sign_tx(tx)
    #     account.unlock(password)



from kivy.app import App
from PIL import Image as PILImage

from etherollapp.etheroll.ui_utils import SubScreen, load_kv_from_py

load_kv_from_py(__file__)

# monkey patching PIL, until it gets monkey patched upstream, refs:
# https://github.com/kivy/kivy/issues/5460
# and refs:
# https://github.com/AndreMiras/PyWallet/issues/104
try:
    # Pillow
    PILImage.frombytes
    PILImage.Image.tobytes
except AttributeError:
    # PIL
    PILImage.frombytes = PILImage.frombuffer
    PILImage.Image.tobytes = PILImage.Image.tostring


class FlashQrCodeScreen(SubScreen):

    @property
    def zbarcam(self):
        return self.ids.zbarcam_id

    def bind_on_symbols(self):
        """
        Since the camera doesn't seem to stop properly, we always bind/unbind
        on_pre_enter/on_pre_leave.
        """
        self.zbarcam.bind(symbols=self.on_symbols)

    def unbind_on_symbols(self):
        """
        Since the camera doesn't seem to stop properly, makes sure at least
        events are unbound.
        """
        self.zbarcam.unbind(symbols=self.on_symbols)

    def on_symbols(self, instance, symbols):
        # also ignores if more than 1 code were found since we don't want to
        # send to the wrong one
        if len(symbols) != 1:
            return
        symbol = symbols[0]
        # update Send screen address
        self.on_back()
        controller = App.get_running_app().root
        switch_account_screen = controller.switch_account_screen
        send_screen = switch_account_screen.ids.send_id
        send_screen.ids.send_to_id.text = symbol.data.decode('utf-8')

    def on_back(self):
        controller = App.get_running_app().root
        controller.load_switch_account()



from kivy.uix.screenmanager import ScreenManager


class LazyScreenManager(ScreenManager):

    __events__ = ('on_pre_add_widget', 'on_add_widget')

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self._screen_types = {}

    def on_add_widget(self, screen):
        pass

    def on_pre_add_widget(self, screen):
        pass

    def register_screen(self, cls, name):
        self._screen_types[name] = cls

    def on_current(self, instance, value):
        # creates the Screen object if it doesn't exist
        if not self.has_screen(value):
            screen = self._screen_types[value](name=value)
            self.add_widget(screen)
        super().on_current(instance, value)

    def add_widget(self, screen):
        """
        Overrides `ScreenManager.add_widget()` dispatches `on_pre_add_widget`
        and `on_add_widget` events.
        """
        self.dispatch('on_pre_add_widget', screen)
        super().add_widget(screen)
        self.dispatch('on_add_widget', screen)



#!/usr/bin/env python
"""
Roll polling service script.
Monitors the chain on regular basis and send notifications on change.
Also updates the App UI via OSC:
MonitorRollsService -> OscAppClient -> OscAppServer -> App

On Linux run with:
```sh
PYTHONPATH=src/
PYTHON_SERVICE_ARGUMENT='{"osc_server_port": PORT}'
./src/etherollapp/service/main.py
```
"""
import json
import os
from time import sleep, time

from kivy.app import App
from kivy.logger import Logger
from kivy.utils import platform
from plyer import notification
from pyetheroll.constants import ROUND_DIGITS
from pyetheroll.etheroll import Etheroll
from raven import Client

from etherollapp.ethereum_utils import AccountUtils
from etherollapp.etheroll.constants import API_KEY_PATH
from etherollapp.etheroll.settings import Settings
from etherollapp.osc.osc_app_client import OscAppClient
from etherollapp.sentry_utils import configure_sentry

PULL_FREQUENCY_SECONDS = 10
# time before the service shuts down if no roll activity
NO_ROLL_ACTIVITY_PERDIOD_SECONDS = 5 * 60


class EtherollApp(App):

    @staticmethod
    def get_files_dir():
        """
        Alternative App._get_user_data_dir() implementation for Android
        that also works when within a service activity.
        """
        from jnius import autoclass, cast
        PythonActivity = autoclass('org.kivy.android.PythonActivity')
        activity = PythonActivity.mActivity
        if activity is None:
            # assume we're running from the background service
            PythonService = autoclass('org.kivy.android.PythonService')
            activity = PythonService.mService
        context = cast('android.content.Context', activity)
        file_p = cast('java.io.File', context.getFilesDir())
        data_dir = file_p.getAbsolutePath()
        return data_dir

    def _get_user_data_dir(self):
        """
        Overrides the default `App._get_user_data_dir()` behavior on Android to
        also work with service activity.
        """
        if platform == 'android':
            return self.get_files_dir()
        return super()._get_user_data_dir()


class MonitorRollsService():

    def __init__(self, osc_server_port=None):
        """
        Set `osc_server_port` to enable UI synchronization with service.
        """
        self._pyetheroll = None
        self._account_utils = None
        # per address cached merged logs, used to compare with next pulls
        self.merged_logs = {}
        self.last_roll_activity = None
        self.osc_app_client = None
        if osc_server_port is not None:
            self.osc_app_client = OscAppClient('localhost', osc_server_port)
        # so that the `App._running_app` singleton is available
        EtherollApp()

    def run(self):
        """
        Blocking pull loop call.
        Service will stop after a period of time with no roll activity.
        """
        self.last_roll_activity = time()
        elapsed = (time() - self.last_roll_activity)
        while elapsed < NO_ROLL_ACTIVITY_PERDIOD_SECONDS:
            self.pull_accounts_rolls()
            sleep(PULL_FREQUENCY_SECONDS)
            elapsed = (time() - self.last_roll_activity)
        # service decided to die naturally after no roll activity
        self.set_auto_restart_service(False)

    @property
    def account_utils(self):
        """
        Gets or creates the AccountUtils object so it loads lazily.
        """
        keystore_dir = Settings.get_keystore_path()
        return AccountUtils.get_or_create(keystore_dir)

    @staticmethod
    def set_auto_restart_service(restart=True):
        """
        Makes sure the service restarts automatically on Android when killed.
        """
        if platform != 'android':
            return
        from jnius import autoclass
        PythonService = autoclass('org.kivy.android.PythonService')
        PythonService.mService.setAutoRestartService(restart)

    @property
    def pyetheroll(self):
        """
        Gets or creates the Etheroll object.
        Also recreates the object if the chain_id changed.
        """
        chain_id = Settings.get_stored_network()
        print(f'chain_id: {chain_id}')
        if self._pyetheroll is None or self._pyetheroll.chain_id != chain_id:
            self._pyetheroll = Etheroll(API_KEY_PATH, chain_id)
        return self._pyetheroll

    def pull_account_rolls(self, account):
        """
        Retrieves the merged logs for the given account and compares it with
        existing cached value. If it differs notifies and updates cache.
        """
        address = "0x" + account.address.hex()
        merged_logs = self.pyetheroll.get_merged_logs(address=address)
        try:
            merged_logs_cached = self.merged_logs[address]
        except KeyError:
            # not yet cahed, let's cache it for the first time
            self.merged_logs[address] = merged_logs
            return
        if merged_logs_cached != merged_logs:
            # since it differs, updates the cache and notifies
            self.merged_logs[address] = merged_logs
            self.do_notify(merged_logs)
            self.last_roll_activity = time()

    def pull_accounts_rolls(self):
        accounts = []
        accounts = self.account_utils.get_account_list()
        print(f'accounts: {accounts}')
        for account in accounts:
            self.pull_account_rolls(account)

    def do_notify(self, merged_logs):
        """
        Notifies the with last roll.
        If the roll has no bet result, notifies it was just placed on the
        blockchain, but not yet resolved by the oracle.
        If it has a result, notifies it.
        Also notifies the app process via OSC so it can refresh balance.
        """
        merged_log = merged_logs[-1]
        bet_log = merged_log['bet_log']
        bet_result = merged_log['bet_result']
        bet_value_ether = bet_log['bet_value_ether']
        roll_under = bet_log['roll_under']
        ticker = "Ticker"
        # the bet was just placed, but not resolved by the oracle
        if bet_result is None:
            title = "Bet confirmed on chain"
            message = (
                '{bet_value_ether:.{round_digits}f} ETH '
                'to roll under {roll_under}').format(**{
                    'bet_value_ether': bet_value_ether,
                    'round_digits': ROUND_DIGITS,
                    'roll_under': roll_under})
        else:
            dice_result = bet_result['dice_result']
            player_won = dice_result < roll_under
            sign = '<' if player_won else '>'
            title = 'You '
            title += 'won' if player_won else 'lost'
            message = '{0} {1} {2}'.format(
                dice_result, sign, roll_under)
        kwargs = {'title': title, 'message': message, 'ticker': ticker}
        if self.osc_app_client is not None:
            self.osc_app_client.send_refresh_balance()
        notification.notify(**kwargs)


def main():
    # only send Android errors to Sentry
    in_debug = platform != "android"
    client = configure_sentry(in_debug)
    argument = os.environ.get('PYTHON_SERVICE_ARGUMENT', 'null')
    argument = json.loads(argument)
    argument = {} if argument is None else argument
    osc_server_port = argument.get('osc_server_port')
    service = MonitorRollsService(osc_server_port)
    try:
        service.set_auto_restart_service()
        service.run()
    except Exception:
        # avoid auto-restart loop
        service.set_auto_restart_service(False)
        if type(client) == Client:
            Logger.info(
                'Errors will be sent to Sentry, run with "--debug" if you '
                'are a developper and want to the error in the shell.')
        client.captureException()


if __name__ == '__main__':
    main()



import os

from kivy.app import App
from kivy.storage.jsonstore import JsonStore


class Store:

    @classmethod
    def get_store_path(cls):
        """
        Returns the full user store path.
        On Android, the store is purposely not stored on the sdcard.
        That way we don't need permission for handling user settings.
        Also losing it is not critical.
        """
        app = App.get_running_app()
        return os.path.join(app.user_data_dir, 'store.json')

    @classmethod
    def get_store(cls):
        """Returns user Store object."""
        store_path = cls.get_store_path()
        store = JsonStore(store_path)
        return store



import shutil
import unittest
from tempfile import mkdtemp
from unittest import mock

from kivy.app import App
from pyetheroll.constants import ChainID

from etherollapp.etheroll.settings import Settings
from etherollapp.service.main import EtherollApp


class TestSettings(unittest.TestCase):
    """Unit tests Settings methods."""
    @classmethod
    def setUpClass(cls):
        EtherollApp()

    def setUp(self):
        """Creates a temporary user data dir for storing the user config."""
        self.temp_path = mkdtemp(prefix='etheroll')
        self.app = App.get_running_app()
        self.app._user_data_dir = self.temp_path

    def tearDown(self):
        """Deletes temporary user data dir."""
        shutil.rmtree(self.temp_path, ignore_errors=True)

    def test_get_set_stored_network(self):
        """Checks default stored network and set method."""
        # checks default
        assert Settings.get_stored_network() == ChainID.MAINNET
        # checks set
        Settings.set_stored_network(ChainID.ROPSTEN)
        assert Settings.get_stored_network() == ChainID.ROPSTEN

    def test_is_stored_mainnet(self):
        Settings.set_stored_network(ChainID.MAINNET)
        assert Settings.is_stored_mainnet() is True
        Settings.set_stored_network(ChainID.ROPSTEN)
        assert Settings.is_stored_mainnet() is False

    def test_is_stored_testnet(self):
        Settings.set_stored_network(ChainID.MAINNET)
        assert Settings.is_stored_testnet() is False
        Settings.set_stored_network(ChainID.ROPSTEN)
        assert Settings.is_stored_testnet() is True

    def test_get_set_stored_gas_price(self):
        """Checks default stored gas price and set method."""
        # checks default
        assert Settings.get_stored_gas_price() == 4
        # checks set
        Settings.set_stored_gas_price(42)
        assert Settings.get_stored_gas_price() == 42

    def test_get_set_is_persistent_keystore(self):
        """Checks default persist value and set method."""
        # checks default
        assert Settings.is_persistent_keystore() is False
        # checks set
        Settings.set_is_persistent_keystore(True)
        assert Settings.is_persistent_keystore() is True

    def test_get_android_keystore_prefix(self):
        """
        The keystore prefix should be the same as user_data_dir by default.
        But it can also be persisted to the sdcard.
        """
        assert Settings.is_persistent_keystore() is False
        prefix = Settings._get_android_keystore_prefix()
        assert prefix == self.app.user_data_dir
        with mock.patch.object(
                Settings, 'is_persistent_keystore', return_value=True):
            prefix = Settings._get_android_keystore_prefix()
        assert prefix == '/sdcard/etheroll'



import os
import shutil

from kivy.properties import BooleanProperty, NumericProperty
from pyetheroll.constants import ChainID

from etherollapp.etheroll.constants import KEYSTORE_DIR_SUFFIX
from etherollapp.etheroll.settings import Settings
from etherollapp.etheroll.ui_utils import SubScreen, load_kv_from_py
from etherollapp.etheroll.utils import (check_request_write_permission,
                                        check_write_permission)

load_kv_from_py(__file__)


class SettingsScreen(SubScreen):
    """Screen for configuring network, gas price..."""

    is_stored_mainnet = BooleanProperty()
    is_stored_testnet = BooleanProperty()
    stored_gas_price = NumericProperty()

    def store_network(self):
        """Saves selected network to the store."""
        network = self.get_ui_network()
        Settings.set_stored_network(network)

    def store_gas_price(self):
        """Saves gas price value to the store."""
        gas_price = self.get_ui_gas_price()
        Settings.set_stored_gas_price(gas_price)

    def store_is_persistent_keystore(self):
        """
        Saves the persistency option to the store.
        Note that to save `True` we also check if we have write permissions.
        """
        persist_keystore = self.is_ui_persistent_keystore()
        persist_keystore = persist_keystore and check_write_permission()
        persistency_toggled = (
            Settings.is_persistent_keystore() != persist_keystore)
        if persistency_toggled:
            self.sync_keystore(persist_keystore)
        Settings.set_is_persistent_keystore(persist_keystore)

    def sync_to_directory(source_dir, destination_dir):
        """
        Copy source dir content to the destination dir one.
        Files already existing get overriden.
        """
        os.makedirs(destination_dir, exist_ok=True)
        files = os.listdir(source_dir)
        for f in files:
            source_file = os.path.join(source_dir, f)
            # file path is given rather than the dir so it gets overriden
            destination_file = os.path.join(destination_dir, f)
            try:
                shutil.copy(source_file, destination_file)
            except PermissionError:
                # `copymode()` may have fail, fallback to simple copy
                shutil.copyfile(source_file, destination_file)

    @classmethod
    def sync_keystore_to_persistent(cls):
        """Copies keystore from non persistent to persistent storage."""
        # TODO: handle dir doesn't exist
        source_dir = os.path.join(
            Settings.get_non_persistent_keystore_path(),
            KEYSTORE_DIR_SUFFIX)
        destination_dir = os.path.join(
            Settings.get_persistent_keystore_path(),
            KEYSTORE_DIR_SUFFIX)
        cls.sync_to_directory(source_dir, destination_dir)

    @classmethod
    def sync_keystore_to_non_persistent(cls):
        """Copies keystore from persistent to non persistent storage."""
        # TODO: handle dir doesn't exist
        source_dir = os.path.join(
            Settings.get_persistent_keystore_path(),
            KEYSTORE_DIR_SUFFIX)
        destination_dir = os.path.join(
            Settings.get_non_persistent_keystore_path(),
            KEYSTORE_DIR_SUFFIX)
        cls.sync_to_directory(source_dir, destination_dir)

    @classmethod
    def sync_keystore(cls, to_persistent):
        if to_persistent:
            cls.sync_keystore_to_persistent()
        else:
            cls.sync_keystore_to_non_persistent()

    def set_persist_keystore_switch_state(self, active):
        """
        The MDSwitch UI look doesn't seem to be binded to its status.
        Here the UI look will be updated depending on the "active" status.
        """
        mdswitch = self.ids.persist_keystore_switch_id
        if self.is_ui_persistent_keystore() != active:
            mdswitch.ids.thumb.trigger_action()

    def load_settings(self):
        """Load json store settings to UI properties."""
        self.is_stored_mainnet = Settings.is_stored_mainnet()
        self.is_stored_testnet = Settings.is_stored_testnet()
        self.stored_gas_price = Settings.get_stored_gas_price()
        is_persistent_keystore = (
            Settings.is_persistent_keystore() and check_write_permission())
        self.set_persist_keystore_switch_state(is_persistent_keystore)

    def store_settings(self):
        """Stores settings to json store."""
        self.store_gas_price()
        self.store_network()
        self.store_is_persistent_keystore()

    def get_ui_network(self):
        """Retrieves network values from UI."""
        if self.is_ui_mainnet():
            network = ChainID.MAINNET
        else:
            network = ChainID.ROPSTEN
        return network

    def is_ui_mainnet(self):
        return self.ids.mainnet_checkbox_id.active

    def is_ui_testnet(self):
        return self.ids.testnet_checkbox_id.active

    def get_ui_gas_price(self):
        return self.ids.gas_price_slider_id.value

    def is_ui_persistent_keystore(self):
        return self.ids.persist_keystore_switch_id.active

    def check_request_write_permission(self):
        # previous state before the toggle
        if self.is_ui_persistent_keystore():
            check_request_write_permission()



import os

from setuptools import find_namespace_packages, setup

from src.etherollapp import version


def read(fname):
    with open(os.path.join(os.path.dirname(__file__), fname)) as f:
        return f.read()


# exposing the params so it can be imported
setup_params = {
    'name': 'EtherollApp',
    'version': version.__version__,
    'description': (
        'Provably fair dice game running on the Ethereum blockchain'),
    'long_description': read('README.md'),
    'long_description_content_type': 'text/markdown',
    'author': 'Andre Miras',
    'url': 'https://github.com/AndreMiras/EtherollApp',
    'packages': find_namespace_packages(
        where='src',
        exclude=(
            'distutils',
            'python-for-android',
            'python-for-android.*',
        ),
    ),
    'package_data': {'': ('*.kv', '*.md')},
    'package_dir': {'': 'src'},
    'entry_points': {
        'console_scripts': (
            'etherollapp=etherollapp.etheroll.controller:main',
        ),
    },
    'python_requires': '>=3',
    'install_requires': (
        'eth-account<0.4.0',
        'eth-utils',
        'kivy-garden.kivymd',
        'layoutmargin',
        'oscpy',
        'pyetheroll',
        'raven',
        'requests-cache',
        'web3',
    ),
}


def run_setup():
    setup(**setup_params)


# makes sure the setup doesn't run at import time
if __name__ == '__main__':
    run_setup()



import re

from kivy.properties import ObjectProperty
from kivy.uix.boxlayout import BoxLayout
from kivymd.textfields import MDTextField

from etherollapp.etheroll.ui_utils import load_kv_from_py

load_kv_from_py(__file__)


def is_number(s):
    try:
        float(s)
    except ValueError:
        return False
    return True


class MDFloatInput(MDTextField):

    pat = re.compile('[^0-9]')

    def insert_text(self, substring, from_undo=False):
        pat = self.pat
        if '.' in self.text:
            s = re.sub(pat, '', substring)
        else:
            s = '.'.join([re.sub(pat, '', s) for s in substring.split('.', 1)])
        return super().insert_text(s, from_undo=from_undo)


class Send(BoxLayout):

    current_account = ObjectProperty(allownone=True)
    __events__ = ('on_send',)

    def on_send(self, address, amount_eth):
        pass

    def on_send_release(self):
        address = self.ids.send_to_id.text
        amount_eth = float(self.ids.amount_eth_id.text)
        self.dispatch('on_send', address, amount_eth)



from etherscan.client import ConnectionRefused
from kivy.app import App
from kivy.clock import Clock, mainthread
from kivy.properties import NumericProperty, StringProperty
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.gridlayout import GridLayout
from kivy.uix.screenmanager import Screen
from pyetheroll.constants import ROUND_DIGITS
from requests.exceptions import ConnectionError

from etherollapp.etheroll.ui_utils import Dialog, load_kv_from_py
from etherollapp.etheroll.utils import run_in_thread

load_kv_from_py(__file__)
DEFAULT_MIN_BET = 0.10


class RollUnderRecap(GridLayout):
    roll_under_property = NumericProperty()
    profit_property = NumericProperty()
    wager_property = NumericProperty()


class BetSize(BoxLayout):
    min_bet_property = NumericProperty(DEFAULT_MIN_BET)

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        Clock.schedule_once(self._after_init)

    def _after_init(self, dt):
        """Binds events."""
        slider = self.ids.bet_size_slider_id
        inpt = self.ids.bet_size_input_id
        cast_to = float
        # shows less digits than the constant default to keep the input tiny
        round_digits = 2
        BetSize.bind_slider_input(slider, inpt, cast_to, round_digits)
        self.pull_min_bet_from_contract()

    @run_in_thread
    def pull_min_bet_from_contract(self):
        """
        Retrieves (async) the minimum bet size by calling the contract.
        On network error fallback to default minimum bet.
        """
        controller = App.get_running_app().root
        pyetheroll = controller.pyetheroll
        min_bet = DEFAULT_MIN_BET
        try:
            min_bet_wei = pyetheroll.contract.functions.minBet().call()
            min_bet = round(min_bet_wei / 1e18, ROUND_DIGITS)
        except ConnectionError:
            pass
        self.set_min_bet(min_bet)

    @mainthread
    def set_min_bet(self, min_bet):
        self.min_bet_property = min_bet

    @staticmethod
    def bind_slider_input(
            slider, inpt, cast_to=float, round_digits=ROUND_DIGITS):
        """Binds slider <-> input both ways."""
        # slider -> input
        slider.bind(
            value=lambda instance, value:
            setattr(inpt, 'text', "{0:.{1}f}".format(
                cast_to(value), round_digits)))
        # input -> slider
        inpt.bind(
            on_text_validate=lambda instance:
            setattr(slider, 'value', cast_to(inpt.text)))
        # also when unfocused
        inpt.bind(
            focus=lambda instance, focused:
            inpt.dispatch('on_text_validate')
                if not focused else False)
        # synchronises values slider <-> input once
        inpt.dispatch('on_text_validate')

    @property
    def value(self):
        """Returns normalized bet size value."""
        try:
            return round(
                float(self.ids.bet_size_input_id.text), ROUND_DIGITS)
        except ValueError:
            return 0


class ChanceOfWinning(BoxLayout):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        Clock.schedule_once(self._after_init)

    def _after_init(self, dt):
        """Binds events."""
        slider = self.ids.chances_slider_id
        inpt = self.ids.chances_input_id
        cast_to = self.cast_to
        round_digits = 0
        BetSize.bind_slider_input(slider, inpt, cast_to, round_digits)

    @staticmethod
    def cast_to(value):
        return int(float(value))

    @property
    def value(self):
        """Returns normalized chances value."""
        try:
            # `input_filter: 'int'` only verifies that we have a number
            # but doesn't convert to int
            chances = float(self.ids.chances_input_id.text)
            return int(chances)
        except ValueError:
            return 0


class RollScreen(Screen):

    current_account_string = StringProperty(allownone=True)
    balance_property = NumericProperty()

    def get_roll_input(self):
        """Returns bet size and chance of winning user input values."""
        bet_size = self.ids.bet_size_id
        chance_of_winning = self.ids.chance_of_winning_id
        return {
            "bet_size": bet_size.value,
            "chances": chance_of_winning.value,
        }

    @mainthread
    def toggle_widgets(self, enabled):
        """Enables/disables widgets (useful during roll)."""
        self.disabled = not enabled

    @property
    def pyetheroll(self):
        """
        We want to make sure we go through the `Controller.pyetheroll` property
        each time, because it recreates the Etheroll object on chain_id
        changes.
        """
        controller = App.get_running_app().root
        return controller.pyetheroll

    @mainthread
    def update_balance(self, balance):
        """Updates the property from main thread."""
        self.balance_property = balance

    @staticmethod
    @mainthread
    def on_connection_refused():
        title = 'No network'
        body = 'No network, could not retrieve account balance.'
        dialog = Dialog.create_dialog(title, body)
        dialog.open()

    @run_in_thread
    def fetch_update_balance(self):
        """Retrieves the balance and updates the property."""
        address = self.current_account_string
        if not address:
            return
        try:
            balance = self.pyetheroll.get_balance(address)
        except ConnectionRefused:
            self.on_connection_refused()
            return
        self.update_balance(balance)



from kivy.properties import StringProperty
from kivy.uix.boxlayout import BoxLayout

from etherollapp.etheroll.ui_utils import Dialog, load_kv_from_py

load_kv_from_py(__file__)


class PasswordForm(BoxLayout):
    __events__ = ('on_unlock',)
    password = StringProperty()

    def on_unlock(self, instance, account, password):
        pass

    @classmethod
    def dialog(cls, account):
        """Prompt the password dialog."""
        title = "Enter your password"
        password_form = cls()
        password_form.ids.account_id.text = "0x" + account.address.hex()
        dialog = Dialog.create_dialog_content_helper(
                    title=title,
                    content=password_form)
        # workaround for MDDialog container size (too small by default)
        dialog.ids.container.size_hint_y = 1
        dialog.add_action_button(
            "Unlock",
            action=lambda *x: password_form.dispatch(
                'on_unlock', dialog, account, password_form.password))
        # hitting enter on the text should also submit
        password_form.ids.password_id.bind(
            on_text_validate=lambda *x: password_form.dispatch(
                'on_unlock', dialog, account, password_form.password))
        return dialog
